//
//  Button.m
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "Button.h"


@implementation Button

- (Button*)init
{
	self = [super init];
	
	[self setImageOn:nil
			imageOff:nil];
	
	inside = NO;
	
	return self;
}

+ (Button*)initWithID:(int)newID position:(NSRect)newRect text:(NSString*)newText
{
	Button* thisButton = [[[Button alloc] init] autorelease];
	
	[thisButton setButtonID:newID];
	[thisButton setRect:newRect];
	[thisButton setText:newText];
	[thisButton setSelected:NO];
	
	[thisButton setEnabled:YES];
	
	return thisButton;
}

- (void)setImageOn:(NSImage*)newOnImage imageOff:(NSImage*)newOffImage
{
	id old = imageOn;
	imageOn = [newOnImage retain];
	[old release];
	
	old = imageOff;
	imageOff = [newOffImage retain];
	[old release];
}

- (NSImage*)imageOn
{
	return imageOn;
}

- (void)setButtonID:(int)newID
{
	buttonID = newID;
}

- (int)buttonID
{
	return buttonID;
}

- (void)setRect:(NSRect)newRect
{
	rect.origin.x = newRect.origin.x;
	rect.origin.y = newRect.origin.y;
	rect.size.height = newRect.size.height;
	rect.size.width = newRect.size.width;
}

- (NSRect)rect
{
	return rect;
}

- (void)setText:(NSString*)newText
{
	id old = text;
	text = [newText retain];
	[old release];
}

- (NSString*)text
{
	return text;
}

-(void)setEnabled:(BOOL)flag
{
	enabled = flag;
}

- (BOOL)enabled
{
	return enabled;
}

- (void)setData:(id)newData
{
	data = newData;
}

- (id)data
{
	return data;
}

- (void)setInside:(BOOL)flag
{
	inside = flag;
}

- (BOOL)inside
{
	return inside;
}

- (void)setSelected:(BOOL)newSelected
{
	selected = newSelected;
}

- (BOOL)selected
{
	return selected;
}

- (void)drawSelected
{
    NSMutableDictionary *attrText = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [attrText setObject:[NSColor blackColor] forKey:NSForegroundColorAttributeName];
	
	NSRect messageRect;
	
    // set the rectangle for the card
    messageRect.origin.x =  0;
    messageRect.origin.y = 395;
    messageRect.size.width = 640;
    messageRect.size.height = 80;
	
	
    NSPoint messagePoint = NSMakePoint(rect.origin.x + ((rect.size.width - [text sizeWithAttributes:attrText].width) / 2),
									   rect.origin.y + ((rect.size.height - [text sizeWithAttributes:attrText].height) / 2));
	
    [text drawAtPoint:messagePoint withAttributes:attrText];
	
}

- (void)displayButton
{	
	if(imageOn == nil)
	{
		if(![self enabled])
		{
			[[NSColor grayColor] set];
		}
		else
		{
			if(inside)
			{
				//[[NSColor blueColor] set];
				[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.9 alpha:1.0] colorWithAlphaComponent:0.8] set];
			}
			else
			{
				//[[NSColor greenColor] set];
				[[[NSColor colorWithDeviceRed:0.8 green:0.4 blue:0.2 alpha:1.0] colorWithAlphaComponent:0.8] set];
			}
		}
		
				
		NSRectFill([self rect]);
		
		[self drawSelected];
	}
	
	else
	{
		if(inside && [self enabled])
		{
			[imageOn compositeToPoint:NSMakePoint(rect.origin.x, rect.origin.y) operation:NSCompositeSourceOver];

		}
		else
		{
			[imageOff  compositeToPoint:NSMakePoint(rect.origin.x, rect.origin.y) operation:NSCompositeSourceOver];
		}
	}
	
}

- (SEL)method
{
	return method;
}

- (void)setMethod:(SEL)newMethod
{
	method = newMethod;
}

-(void)setNumParams:(int)newValue
{
	numParams = newValue;
}

-(int)numParams
{
	return numParams;
}

@end
