//
//  Distance.m
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "Distance.h"
#import "Factory.h"
#import "City.h"


@implementation Distance

- (id) init
{
	self = [super init];
	
	nodesArrayRoad = [[NSMutableArray alloc] initWithCapacity:0];
	nodesArrayRail = [[NSMutableArray alloc] initWithCapacity:0];
	nodesArrayCanal = [[NSMutableArray alloc] initWithCapacity:0];
	
	built[0] = 0;
	built[1] = 0;
	built[2] = 0;
	
	completed[0] = NO;
	completed[1] = NO;
	completed[2] = NO;
	
	construction[0] = NO;
	construction[1] = NO;
	construction[2] = NO;

	
    return self;
}

- (void) dealloc
{	
	
	if(nodesArrayRoad)
	{
		[nodesArrayRoad release];
	}
	
	if(nodesArrayRail)
	{
		[nodesArrayRail release];
	}

	
	if(nodesArrayCanal)
	{
		[nodesArrayCanal release];
	}
	
    
	[super dealloc];
}

- (void)setFactory:(Factory*)newFactory
{
	factory = newFactory;
	
	// set first node to equal factory location
	[self addNode:[newFactory positionMap] forType:NETWORK_ROAD];
	[self addNode:[newFactory positionMap] forType:NETWORK_RAIL];
	[self addNode:[newFactory positionMap] forType:NETWORK_CANAL];

}

- (Factory*)factory
{
	return factory;
}

- (void)setCity:(City*)newCity
{
	city = newCity;
	
	// add a node in the middle of the points
	float nodeX1, nodeY1;
	float nodeX2, nodeY2;
	
	float deltaX, deltaY;
	
	nodeX1 = [self getNodeAtIndex:0 forType:NETWORK_ROAD].x;
	nodeY1 = [self getNodeAtIndex:0 forType:NETWORK_ROAD].y;
	
	nodeX2 = [newCity positionMap].x;
	nodeY2 = [newCity positionMap].y;
	
	deltaX = (nodeX2 - nodeX1) / [self distance:NETWORK_ROAD];
	deltaY = (nodeY2 - nodeY1) / [self distance:NETWORK_ROAD];

	int loop;
	
	for(loop = 0; loop < [self distance:NETWORK_ROAD] - 1; loop++)
	{
		nodeX1 = nodeX1 + deltaX;
		nodeY1 = nodeY1 + deltaY;
		
		[self addNode:NSMakePoint(nodeX1, nodeY1)forType:NETWORK_ROAD];
	}
	
	// set last node to equal city location
	[self addNode:[newCity positionMap]forType:NETWORK_ROAD];
	
	nodeX1 = [self getNodeAtIndex:0 forType:NETWORK_CANAL].x;
	nodeY1 = [self getNodeAtIndex:0 forType:NETWORK_CANAL].y;
	
	nodeX2 = [newCity positionMap].x;
	nodeY2 = [newCity positionMap].y;
	
	deltaX = (nodeX2 - nodeX1) / [self distance:NETWORK_CANAL];
	deltaY = (nodeY2 - nodeY1) / [self distance:NETWORK_CANAL];
		
	for(loop = 0; loop < [self distance:NETWORK_CANAL] - 1; loop++)
	{
		nodeX1 = nodeX1 + deltaX;
		nodeY1 = nodeY1 + deltaY;
		
		[self addNode:NSMakePoint(nodeX1 - 20 , nodeY1 - 10)forType:NETWORK_CANAL];
	}
	
	// set last node to equal city location
	[self addNode:[newCity positionMap]forType:NETWORK_CANAL];
	
	nodeX1 = [self getNodeAtIndex:0 forType:NETWORK_RAIL].x;
	nodeY1 = [self getNodeAtIndex:0 forType:NETWORK_RAIL].y;
	
	nodeX2 = [newCity positionMap].x;
	nodeY2 = [newCity positionMap].y;
	
	deltaX = (nodeX2 - nodeX1) / [self distance:NETWORK_RAIL];
	deltaY = (nodeY2 - nodeY1) / [self distance:NETWORK_RAIL];
		
	for(loop = 0; loop < [self distance:NETWORK_RAIL] - 1; loop++)
	{
		nodeX1 = nodeX1 + deltaX;
		nodeY1 = nodeY1 + deltaY;
		
		[self addNode:NSMakePoint(nodeX1 + 20, nodeY1 + 10)forType:NETWORK_RAIL];
	}
	
	// set last node to equal city location
	[self addNode:[newCity positionMap]forType:NETWORK_RAIL];

}

- (City*)city
{
	return city;
}

- (void)setDistance:(int)newDistance forType:(int)type
{
	distance[type] = newDistance;
}

- (int)distance:(int)type
{
	return distance[type];
}

- (void)setBuilt:(int)newDistance forType:(int)type
{
	built[type] = newDistance;
}

- (int)built:(int)type
{
	return built[type];
}

- (void)addNode:(NSPoint)newNode forType:(int)type
{
	switch(type)
	{
		case NETWORK_ROAD:
			[nodesArrayRoad addObject:[NSValue valueWithPoint:newNode]];
			break;
			
		case NETWORK_RAIL:
			[nodesArrayRail addObject:[NSValue valueWithPoint:newNode]];
			break;
			
		case NETWORK_CANAL:
			[nodesArrayCanal addObject:[NSValue valueWithPoint:newNode]];
			break;
	}
}

- (int)nodesCount:(int)type
{
	switch(type)
	{
		case NETWORK_ROAD:
			return [nodesArrayRoad count];
			break;
			
		case NETWORK_RAIL:
			return [nodesArrayRail count];
			break;
			
		case NETWORK_CANAL:
			return [nodesArrayCanal count];
			break;
	}
}

- (NSPoint)getNodeAtIndex:(int)index forType:(int)type
{
	switch(type)
	{
		case NETWORK_ROAD:
			if(index < 0 || index > [nodesArrayRoad count])
			{
				NSLog(@"Distance:road:getNodeAtIndex: index of %d out of range", index);
			}
			
			return [[nodesArrayRoad objectAtIndex:index] pointValue];
			break;
			
		case NETWORK_RAIL:
			if(index < 0 || index > [nodesArrayRail count])
			{
				NSLog(@"Distance:rail:getNodeAtIndex: index of %d out of range", index);
			}
			
			return [[nodesArrayRail objectAtIndex:index] pointValue];			break;
			
		case NETWORK_CANAL:
			if(index < 0 || index > [nodesArrayCanal count])
			{
				NSLog(@"Distance:canal:getNodeAtIndex: index of %d out of range", index);
			}
			
			return [[nodesArrayCanal objectAtIndex:index] pointValue];			break;
	}
}


- (BOOL)build:(int)type
{
	if([self built:type] < [self distance:type] && [self construction:type])
	{
		[self setBuilt:[self built:type] + 1
			   forType:type];
		
		if([self built:type] == [self distance:type])
		{
			// return that the network has been completed
			completed[type] = YES;
			return YES;
		}
	}
	
	return NO;
}

- (void)setCompleted:(BOOL)flag forType:(int)type
{
	completed[type] = flag;
}

- (BOOL)completed:(int)type
{
	return completed[type];
}

- (void)setConstruction:(BOOL)flag forType:(int)type
{
	construction[type] = flag;
}

- (BOOL)construction:(int)type
{
	return construction[type];
}

@end
