//
//  Factory.m
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "Factory.h"
#import "Player.h"

@implementation Factory

- (id) init
{
	self = [super init];
	
	[self setType:FACTORY_COTTON];
	[self setWorkforce:0];
	[self setWorkforceMax:10];
	[self setOutputRatio:1];
	[self setStockMax:100];
	[self setStock:0];
	[self setRunningCost:5];
	[self setValue:1000];
	[self setMaxSize:5];
	[self setSize:0];
	[self setTurnsToBuild:5];
	[self setOwner:nil];
	
	[self setPositionMap:NSMakePoint(50, 200)];
	
	[self setImage:[NSImage imageNamed:@"vacant_site"]]; 

    return self;
}

- (void)setType:(int)newType
{
	type = newType;
	
	if(size > 0)
	{
		[self setImage:[NSImage imageNamed:@"factory"]]; 

	}
}

- (int)type
{
	return type;
}

- (NSString*)typeAsText
{
	if(size == 0)
	{
		return @"Vacant Site";
	}
	
	switch(type)
	{
		case FACTORY_COTTON:
			return @"Cotton Mill";
			break;
			
		case FACTORY_TIN:
			return @"Tin Mine";
			break;
			
		case FACTORY_POTTERY:
			return @"Pottery";
			break;
	}
	
	return @"Unknown";
}

- (void)setWorkforceMax:(int)newMax
{
	workforceMax = newMax;
}

- (int)workforceMax
{
	return workforceMax;
}

- (void)setWorkforce:(int)newCurrent
{
	workforce = newCurrent;
}

- (int)workforce
{
	return workforce;
}

- (BOOL)changeWorkforce:(int)change
{
	if(workforce + change < 0 || workforce + change > workforceMax)
	{
		return NO;
	}
	
	workforce += change;
	
	return YES;
}

- (void)setOutputRatio:(int)newRatio
{
	outputRatio = newRatio;
}

- (int)outputRatio
{
	return outputRatio;
}

- (void)setStockMax:(int)newMax
{
	stockMax = newMax;
}

- (int)stockMax
{
	return stockMax;
}

- (void)setStock:(int)newStock
{
	stock = newStock;
}

- (int)stock
{
	return stock;
}

- (BOOL)changeStock:(int)change
{
	if(stock + change < 0 || stock + change > stockMax)
	{
		return NO;
	}
	
	stock += change;
	
	return YES;
}

- (void)setRunningCost:(int)newCost
{
	runningCost = newCost;
}

- (int)runningCost
{
	return runningCost;
}

- (BOOL)changeRunngingCost:(int)change
{
	if(runningCost + change < 0)
	{
		return NO;
	}
	
	runningCost += change;
	
	return YES;
}

- (void)setValue:(int)newValue
{
	value = newValue;
}

- (int)value
{
	return value;
}

- (BOOL)changeValue:(int)change
{
	if(value + change < 0)
	{
		return NO;
	}
	
	value += change;
	return YES;
}

- (void)setMaxSize:(int)newSize
{
	sizeMax = newSize;
}

- (int)sizeMax
{
	return sizeMax;
}

- (void)setSize:(int)newSize
{
	size = newSize;
}

- (int)size
{
	return size;
}

- (BOOL)changeSize:(int)change
{
	if(size + change < 0 || size + change > sizeMax)
	{
		return NO;
	}
	
	size += change;
	
	return YES;
}

- (void)setTurnsToBuild:(int)newTurns
{
	turnsToBuild = newTurns;
}

- (int)turnsToBuild
{
	return turnsToBuild;
}

- (void)changeTurnsToBuild:(int)change
{
	turnsToBuild += change;
}

- (NSString*)description
{
	return [NSString stringWithFormat:@"%@", [self typeAsText]
		];
}

- (void)setPositionMap:(NSPoint)newPosition
{
	positionMap = newPosition;
}

- (NSPoint)positionMap
{
	return positionMap;
}


- (void)setName:(NSString*)newName
{
	id old = name;
	name = [newName retain];
	[old release];	
}

- (NSString*)name
{
	//return name;
	
	return [self typeAsText];
	
	//return [NSString stringWithFormat:@"%@:%@", [[self owner] name], [self typeAsText]];
}

-(void)setOwner:(Player*)newOwner
{
	owner = newOwner;
}

-(Player*)owner
{
	return owner;
}

-(void)setImage:(NSImage*)newImage
{
	id old = image;
	image = [newImage retain];
	[old release];
}

-(NSImage*)image
{
	return image;
}


- (void)setPositionIconWheel:(NSPoint)newPosition
{
	positionIconWheel = newPosition;
}

- (NSPoint)positionIconWheel
{
	return positionIconWheel;
}

@end
