/* IRGame */
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import <Cocoa/Cocoa.h>

enum { STATE_INTRO, STATE_TITLE, STATE_CREDITS, STATE_HIGH_SCORE, STATE_PREFS,
	STATE_OPPONENT_SELECTION, STATE_OFFICE, STATE_MAP, STATE_STATS, STATE_SHOP,
	STATE_BUY_FACTORY, STATE_BUY_TRANSPORT,
	STATE_INVENTIONS,
	STATE_HELP,
	STATE_END_TURN,
	STATE_GAME_OVER };

@class IRController, IRGameView;
@class Player;
@class Distance;
@class City;
@class Factory;
@class Invention;
@class Task;
@class ShopItem;
@class Transport;
@class Help;

@interface IRGame : NSObject
{
    IBOutlet IRController *controller;
    IBOutlet IRGameView *view;
	
	int state;		// current game state
	
	NSMutableArray* playersArray; // array of players in the game
	Player* currentPlayer;
	Task* currentTask;
	Transport* currentTransport;	// the currently selected transport
	Factory* currentFactory;	// the currently selected factory
	City* currentCity;			// the currently selected city

	
	NSMutableArray* distancesArray; // array of distances between factories and cities
	NSMutableArray* citiesArray; // array of cities
	NSMutableArray* factoriesArray; // factories owned
	
	NSMutableArray* messageArray;  // text messages to display on end of turn screen
	NSMutableArray* tasksArray;		// array of tasks to be carried out
	
	NSMutableArray* shopArray;		// array of shop items
	
	int currentShopItem;		// current item being displayed in the shop
	int currentInvention;		// current invention being displayed
	
	NSMutableArray* helpArray;
	int currentHelp;		// the currently selected help item

	// random number stuff
	int randLo;
	int randHi;
	int randSeed;
	
	// sounds
	NSSound*	soundPuck;	// puck sound effect
}

- (void) playSound:(int)sound;


- (int)randNum:(int)range;

- (void)setState:(int)newState;
- (int)state;

- (void)newGame:(BOOL)withTraining;
- (void)newQuickStartGame;

- (void)addPlayer:(Player*)newPlayer;
- (Player*)getPlayerAtIndex:(int)index;
- (int)playerCount;

- (void)setCurrentPlayer:(int)index;
- (Player*)currentPlayer;



- (void)endTurn;

- (void)addDistance:(Distance*)newDistance;
- (Distance*)getDistanceAtIndex:(int)index;
- (int)distanceCount;
- (int)distanceBetweenFactory:(Factory*)factory andCity:(City*)city;
- (Distance*)distanceForFactory:(Factory*)factory andCity:(City*)city;


- (void)addCity:(City*)newCity;
- (City*)getCityAtIndex:(int)index;
- (int)cityCount;

-(void)addFactory:(Factory*)newFactory;
-(int)factoryCount;
-(Factory*)getFactoryAtIndex:(int)index;

-(void)addMessage:(NSString*)newMessage;
-(int)messageCount;
-(NSString*)getMessageAtIndex:(int)index;

- (BOOL)canPlayer:(Player*)player invent:(Invention*)invention;

- (NSString*)technologyName:(int)tech;

#pragma mark -
#pragma mark Task Handling
-(void)populateTasks;
-(void)addTask:(Task*)newTask;
-(int)taskCount;
-(Task*)getTaskAtIndex:(int)index;
-(int)getCurrentTaskIndex;

-(void)setCurrentTask:(int)index;
-(Task*)currentTask;
-(BOOL)getNextTask;

#pragma mark -
#pragma mark Shop Handling
- (int)getShopItemCount;
- (void)setCurrentShopItem:(int)newItem;
- (void)changeCurrentShopItem:(int)change;
- (int)currentShopItem;
- (ShopItem*)getShopItemAt:(int)index;
- (ShopItem*)getCurrentShopItem;
- (void)stockShop;
- (void)buyItem;

- (void)buyTransport:(int)type;
- (void)buyFactory:(int)type;

#pragma mark -
#pragma mark Transport Network Handling
- (Distance*)connectionBetweenCity:(City*)city factory:(Factory*)factory;
- (void)commenceConstruction:(int)type;

#pragma mark -
#pragma mark Invention Handling
- (void)setCurrentInvention:(int)newItem;
- (void)changeCurrentInvention:(int)change;
- (int)currentInvention;
- (void)research:(int)type;
- (void)invent;

- (void)populateInventionTree:(Player*)player;

#pragma mark -
#pragma mark Transport Handling
- (void)setCurrentTransport:(Transport*)newTransport;
- (Transport*)currentTransport;
- (void)loadTransport;
- (void)unloadTransport;
- (void)autoReturnTransport:(BOOL)flag;

#pragma mark -
#pragma mark Factory Handling
- (void)setCurrentFactory:(Factory*)newFactory;
- (Factory*)currentFactory;

- (void)upgradeFactory;
- (void)buildFactory:(int)type;

#pragma mark -
#pragma mark City Handling
- (void)setCurrentCity:(City*)newCity;
- (City*)currentCity;

#pragma mark -
#pragma mark XML Handling
- (void)loadXML;

#pragma mark -
#pragma mark Help Handling
-(void)addHelp:(Help*)newHelp;
-(int)helpCount;
-(Help*)getHelpAtIndex:(int)index;
-(void)setCurrentHelp:(int)index;
-(int)currentHelp;
-(void)changeCurrentHelp:(int)change;

- (void)populateHelp;

#pragma mark -
#pragma mark Disaster Handling
- (void)generateDisaster;


- (void)vote;
@end
