//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import "IRGame.h"
#import "IRGameView.h"
#import "IRController.h"
#import "Player.h"
#import "Factory.h"
#import "Transport.h"
#import "Distance.h"
#import "City.h"
#import "Invention.h"
#import "Task.h"
#import "ShopItem.h"
#import "Stats.h"
#import "Help.h"

@implementation IRGame

- (id) init
{
	self = [super init];

	// load the sound effects
	soundPuck = [NSSound soundNamed:@"puck"];
	
	// create the arrays
	playersArray = [[NSMutableArray alloc] initWithCapacity:0];
	distancesArray = [[NSMutableArray alloc] initWithCapacity:0];
	citiesArray = [[NSMutableArray alloc] initWithCapacity:0];
	factoriesArray = [[NSMutableArray alloc] initWithCapacity:0];
	messageArray = [[NSMutableArray alloc] initWithCapacity:0];
	tasksArray = [[NSMutableArray alloc] initWithCapacity:0];
	helpArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	[self populateHelp];
	
	[self loadXML];
	
	currentInvention = 0;
	
	// set random number stuff
	randSeed = 123456;
	randLo = randSeed;
	randHi = ~randSeed;
	

	
	return self;
}

- (void) dealloc
{	
	// release the sounds
	[soundPuck release];

	
	if(playersArray)
	{
		[playersArray release];
	}
	
	if(distancesArray)
	{
		[distancesArray release];
	}
	
	if(citiesArray)
	{
		[citiesArray release];
	}
	
	if(factoriesArray)
	{
		[factoriesArray release];
	}
	
	if(messageArray)
	{
		[messageArray release];
	}
	
	if(tasksArray)
	{
		[tasksArray release];
	}
	
	if(shopArray)
    {
        [shopArray release];
    }
	
	if(helpArray)
	{
		[helpArray release];
	}
		
    [super dealloc];
}

- (int)randNum:(int)range
{
	randHi = (randHi << 16) + (randHi >> 16);
	randHi += randLo;
	randLo += randHi;
	
	NSLog(@"%d in range 0 to %d", (randHi & INT_MAX) % range, range);
	
	return (randHi & INT_MAX) % range;
}

- (void)setState:(int)newState
{
	state = newState;
	
	[view setDisplayState];
	
	if(view)
	{
		[view setTrackingAreas];
	}
}


- (int)state
{
	return state;
}

// start the game but with some default assests
- (void)newQuickStartGame
{
	int loop;
	// set the help system
	[self setCurrentHelp:0];
	
	// remove all existing players
	if(playersArray)
	{
		[playersArray release];
	}
	playersArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all distances lookups
	if(distancesArray)
	{
		[distancesArray release];
	}
	distancesArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all the cities
	if(citiesArray)
	{
		[citiesArray release];
	}
	
	citiesArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all factories
	if(factoriesArray)
	{
		[factoriesArray release];
	}
	
	factoriesArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all existing tasks
	if(tasksArray)
	{
		[tasksArray release];
	}
	
	tasksArray = [[NSMutableArray alloc] initWithCapacity:0];		
	
	// add some cities to the map
	// when using Photoshop co-ords y = ((600 - value) * 1.5) - 100
	City* newCity = nil;
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Manchester"];
	[newCity setPositionMap:NSMakePoint((124 * 1.5) + 50, (323 * 1.5) - 00)]; // sub 50 x add 100 y
	[newCity setPrice:10];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Birmingham"];
	[newCity setPositionMap:NSMakePoint((137 * 1.5) + 200, (246 * 1.5) - 40)];
	[newCity setPrice:10];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Bristol"];
	[newCity setPositionMap:NSMakePoint((127 * 1.5) + 50, (172 * 1.5) - 50)];
	[newCity setPrice:15];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"London"];
	[newCity setPositionMap:NSMakePoint((275 * 1.5) + 220, (181 * 1.5) - 100)];
	[newCity setPrice:20];
	[self addCity:newCity];
	
	
	Player* newPlayer = nil;
	
	// add 4 players
	for(loop = 0; loop < 1; loop++)
	{
		newPlayer = [[[Player alloc] init] retain];
		
		[newPlayer setGame:self];
		
		[newPlayer setName:[NSString stringWithFormat:@"Player %d", loop + 1]];
		[newPlayer setAI:loop == 0 ? 0 : 1];
		[newPlayer setMoney:10000 * 1];
		//[newPlayer setTech:TECH_ROAD
		//			 level:1];
		
		// give player a factory
		Factory* newFactory = [[[Factory alloc] init] retain];
		[newFactory setSize:1];
		[newFactory setWorkforce:[newFactory workforceMax]];
		[newFactory setType:loop % 3];
		[newFactory setTurnsToBuild:0];
		[newFactory setPositionMap:NSMakePoint((124 * 1.5) + 140, (323 * 1.5) - 20)];
		
		[newFactory setOwner:newPlayer];
		[self addFactory:newFactory];
			
		 // give player a horse and cart
		 Transport* newTransport = [[[Transport alloc] init] retain];
		 [newTransport setType:TRANSPORT_CART];
		 [newTransport setLocation:newFactory];
		 [newPlayer addTransport:newTransport];
		
		/*
		 
		 // give player a train
		 newTransport = [[[Transport alloc] init] retain];
		 [newTransport setType:TRANSPORT_TRAIN];
		 [newTransport setLocation:newFactory];
		 [newPlayer addTransport:newTransport];
		 
		 // give player a train
		 newTransport = [[[Transport alloc] init] retain];
		 [newTransport setType:TRANSPORT_BARGE];
		 [newTransport setLocation:newFactory];
		 [newPlayer addTransport:newTransport];
		 */
		
		[self addPlayer:newPlayer];
		
		[self populateInventionTree:newPlayer];

		
	}
	
	[self setCurrentPlayer:0];
	[self setCurrentTask:0];
	
	currentFactory = nil;
	currentTransport = nil;
	currentCity = nil;
	
	
	[self setState:STATE_MAP];
	[view setNeedsDisplay:YES];
}

// Intialise the game world
- (void)newGame:(BOOL)withTraining
{
	int loop;
	// set the help system
	[self setCurrentHelp:0];
	
	// remove all existing players
	if(playersArray)
	{
		[playersArray release];
	}
	playersArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all distances lookups
	if(distancesArray)
	{
		[distancesArray release];
	}
	distancesArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all the cities
	if(citiesArray)
	{
		[citiesArray release];
	}

	citiesArray = [[NSMutableArray alloc] initWithCapacity:0];

	// remove all factories
	if(factoriesArray)
	{
		[factoriesArray release];
	}

	factoriesArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// remove all existing tasks
	if(tasksArray)
	{
		[tasksArray release];
	}
	
	tasksArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	[self stockShop];

	if(withTraining)
	{
		[self populateTasks];
	}
		
	
	// add some cities to the map
	// when using Photoshop co-ords y = ((600 - value) * 1.5) - 100
	City* newCity = nil;
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Manchester"];
	[newCity setPositionMap:NSMakePoint((124 * 1.5) + 50, (323 * 1.5) - 00)]; // sub 50 x add 100 y
	[newCity setPrice:10];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Birmingham"];
	[newCity setPositionMap:NSMakePoint((137 * 1.5) + 200, (246 * 1.5) - 40)];
	[newCity setPrice:10];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"Bristol"];
	[newCity setPositionMap:NSMakePoint((127 * 1.5) + 50, (172 * 1.5) - 50)];
	[newCity setPrice:15];
	[self addCity:newCity];
	
	newCity = [[[City alloc] init] retain];
	[newCity setName:@"London"];
	[newCity setPositionMap:NSMakePoint((275 * 1.5) + 220, (181 * 1.5) - 100)];
	[newCity setPrice:20];
	[self addCity:newCity];
	
		
	Player* newPlayer = nil;
	
	// add 4 players
	for(loop = 0; loop < 1; loop++)
	{
		newPlayer = [[[Player alloc] init] retain];
		
		[newPlayer setGame:self];
		
		[newPlayer setName:[NSString stringWithFormat:@"Player %d", loop + 1]];
		[newPlayer setAI:loop == 0 ? 0 : 1];
		[newPlayer setMoney:10000 * 1];
		//[newPlayer setTech:TECH_ROAD
		//			 level:1];
		
		// give player a factory
		Factory* newFactory = [[[Factory alloc] init] retain];
		[newFactory setType:loop % 3];
		[newFactory setTurnsToBuild:0];
		[newFactory setPositionMap:NSMakePoint((124 * 1.5) + 140, (323 * 1.5) - 20)];
		
		//[newFactory setOwner:newPlayer];
		[self addFactory:newFactory];
		
		
		// give player a second factory
		newFactory = [[[Factory alloc] init] retain];
		[newFactory setType:1 % 3];
		[newFactory setTurnsToBuild:0];
		[newFactory setPositionMap:NSMakePoint((137 * 1.5) + 220, (246 * 1.5) - 200)];
		
		//[newFactory setOwner:newPlayer];
		[self addFactory:newFactory];
		 
		
		
	
		/*
		// give player a horse and cart
		Transport* newTransport = [[[Transport alloc] init] retain];
		[newTransport setType:TRANSPORT_CART];
		[newTransport setLocation:newFactory];
		[newPlayer addTransport:newTransport];
		 */

		/*
		
		// give player a train
		newTransport = [[[Transport alloc] init] retain];
		[newTransport setType:TRANSPORT_TRAIN];
		[newTransport setLocation:newFactory];
		[newPlayer addTransport:newTransport];
		
		// give player a train
		newTransport = [[[Transport alloc] init] retain];
		[newTransport setType:TRANSPORT_BARGE];
		[newTransport setLocation:newFactory];
		[newPlayer addTransport:newTransport];
		*/
		
		
		[self populateInventionTree:newPlayer];
		[self addPlayer:newPlayer];
		
		
				
	}
	
	[self setCurrentPlayer:0];
	[self setCurrentTask:0];
	
	currentFactory = nil;
	currentTransport = nil;
	currentCity = nil;

	
	[self setState:STATE_MAP];
	[view setNeedsDisplay:YES];
}

- (void)addPlayer:(Player*)newPlayer
{
	[playersArray addObject:newPlayer];
}
- (Player*)getPlayerAtIndex:(int)index
{
	return [playersArray objectAtIndex:index];
}

- (int)playerCount
{
	return [playersArray count];
}

- (void)setCurrentPlayer:(int)index
{
	currentPlayer = [self getPlayerAtIndex:index];
}

- (Player*)currentPlayer
{
	return currentPlayer;
}

#pragma mark -
#pragma mark Task Handling
-(void)populateTasks
{
	Task* newTask = nil;

	// create the dictionay for looking up value names
	NSDictionary *valuesDictionary = [NSDictionary dictionaryWithObjectsAndKeys:
		[NSNumber numberWithInt:FACTORY_COTTON],@"FACTORY_COTTON",
		[NSNumber numberWithInt:FACTORY_TIN],@"FACTORY_TIN",
		[NSNumber numberWithInt:FACTORY_POTTERY],@"FACTORY_POTTERY",
		[NSNumber numberWithInt:TRANSPORT_BARGE], @"TRANSPORT_BARGE",
		[NSNumber numberWithInt:TRANSPORT_CART], @"TRANSPORT_CART",
		[NSNumber numberWithInt:TRANSPORT_TRAIN], @"TRANSPORT_TRAIN",
		[NSNumber numberWithInt:TECH_ART], @"TECH_ART",
		[NSNumber numberWithInt:TECH_CONSTRUCTION], @"TECH_CONSTRUCTION",
		[NSNumber numberWithInt:TECH_METAL], @"TECH_METAL",
		[NSNumber numberWithInt:TECH_PUMP], @"TECH_PUMP",
		[NSNumber numberWithInt:TECH_STEAM], @"TECH_STEAM",
		nil];
	
	//  load the training tasks in from an external file
		NSString* tempString;
	int tempValue;
	id tempObj;
	
	NSScanner* valueScanner;
	NSScanner* scanner;	
	
	NSString* fileString = [NSString stringWithContentsOfFile:[NSString stringWithFormat:@"%@/tasks.dat",
		[[NSBundle mainBundle] resourcePath]]];
	
	NSLog([NSString stringWithFormat:@"%@/tasks.dat",
		[[NSBundle mainBundle] resourcePath]]);
	
	NSMutableCharacterSet* letterSet = [[NSCharacterSet letterCharacterSet] mutableCopy];
	NSCharacterSet* whiteSet;
	[letterSet formUnionWithCharacterSet:[NSCharacterSet whitespaceCharacterSet]];
	[letterSet formUnionWithCharacterSet:[NSCharacterSet decimalDigitCharacterSet]];
	[letterSet addCharactersInString:@".,_():"];
	
	NSMutableCharacterSet* defineSet = [[NSCharacterSet letterCharacterSet] mutableCopy];
	[defineSet addCharactersInString:@"_"];


	whiteSet = [NSCharacterSet whitespaceAndNewlineCharacterSet];
	scanner = [NSScanner scannerWithString:fileString];
	
	while(![scanner isAtEnd])
	{
		if([scanner scanString:@"Title:" intoString:nil])
		{
			newTask = [[[Task alloc] init] retain];
			
			[scanner scanCharactersFromSet:letterSet intoString:&tempString];
			[newTask setTitle:[NSString stringWithString:tempString]];
			
			NSLog(@"Title:%@", tempString);
			
			[scanner scanString:@"Overview:" intoString:nil];
			[scanner scanCharactersFromSet:letterSet intoString:&tempString];
			[newTask setOverview:[NSString stringWithString:tempString]];
			
			NSLog(@"Overview:%@", tempString);
			
			[scanner scanString:@"Selector:" intoString:nil];
			[scanner scanCharactersFromSet:letterSet intoString:&tempString];
			NSLog(@"Selector:%@", tempString);
			
			[newTask setCheck:NSSelectorFromString(tempString)];
				
			[scanner scanString:@"Param1:" intoString:nil];
			[scanner scanCharactersFromSet:letterSet intoString:&tempString];
			NSLog(@"Param1:%@", tempString);
			
			if([tempString hasPrefix:@"Number("])
			{
				// check for dictionary value
				valueScanner = [NSScanner scannerWithString:tempString];
				[valueScanner setScanLocation:7];
				
				NSString* tempValueString;
				
				if([valueScanner scanCharactersFromSet:defineSet intoString:&tempValueString])
				{
					NSLog(@"%@", tempValueString);
					
					tempObj = [valuesDictionary objectForKey:tempValueString];
					
					if(tempObj)
					{
						NSNumber* valueNumber = tempObj;
						NSLog(@"= %d", [valueNumber intValue]);
						[newTask setParam1:[NSNumber numberWithInt:[valueNumber intValue]]];

					}
					
				}
				else
				{
					[valueScanner scanInt:&tempValue];
					NSLog(@"Value:%@ = %d", tempString, tempValue);
					[newTask setParam1:[NSNumber numberWithInt:tempValue]];

				}
			}
			
			if([tempString hasPrefix:@"Text("])
			{
				valueScanner = [NSScanner scannerWithString:tempString];
				[valueScanner setScanLocation:5];
				
				NSString* tempValueString;
				
				[valueScanner scanCharactersFromSet:defineSet intoString:&tempValueString];
				
				
				[newTask setParam1:[NSString stringWithString:tempValueString]];
				
			}
			
			[scanner scanString:@"Value:" intoString:nil];
			[scanner scanCharactersFromSet:letterSet intoString:&tempString];
			
			if([tempString hasPrefix:@"Number("])
			{
				valueScanner = [NSScanner scannerWithString:tempString];
				[valueScanner scanString:@"Number(" intoString:nil];
				[valueScanner scanInt:&tempValue];
				NSLog(@"Value:%@ = %d", tempString, tempValue);

				[newTask setValue:[NSNumber numberWithInt:tempValue]];
			}
		
			if([tempString hasPrefix:@"Bool("])
			{
				if([tempString hasPrefix:@"YES"])
				{
					[newTask setValue:[NSNumber numberWithBool:YES]];
				}
				else
				{
					[newTask setValue:[NSNumber numberWithBool:NO]];
				}
			}
			[self addTask:newTask];

			[scanner scanCharactersFromSet:whiteSet intoString:nil];
		}
	}
	
	
/*
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Invent Canal Technolgy"];
	[newTask setOverview:@"Visit the Invention screen and invent the canals."];
	[newTask setCheck:@selector(invented:)];
	[newTask setParam1:@"Canal"];
	[newTask setValue:[NSNumber numberWithBool:YES]];
	
	[self addTask:newTask];

	 
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Build a Cotton Mill"];
	[newTask setOveriview:@"Click on a Vacant Site and the click Build a Cotton Mill to construct your first factory."];
	[newTask setCheck:@selector(factoryCountOfType:)];
	[newTask setParam1:[NSNumber numberWithInt:FACTORY_COTTON]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Buy a Horse and Cart"];
	[newTask setOveriview:@"Visit the shop screen to purchase a Horse and Cart to move your completed units to the city"];
	[newTask setCheck:@selector(transportCountOfType:)];
	[newTask setParam1:[NSNumber numberWithInt:TRANSPORT_CART]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Research Construction"];
	[newTask setOveriview:@"Visit the Invention screen and do some research into construction."];
	[newTask setCheck:@selector(tech:)];
	[newTask setParam1:[NSNumber numberWithInt:TECH_CONSTRUCTION]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Invent Canal Technolgy"];
	[newTask setOveriview:@"Visit the Invention screen and invent the canals."];
	[newTask setCheck:@selector(invented:)];
	[newTask setParam1:@"Canal"];
	[newTask setValue:[NSNumber numberWithBool:YES]];
	
	[self addTask:newTask];
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Buy a Barge"];
	[newTask setOveriview:@"Visit the shop screen to purchase a Barge to move your completed units to the city"];
	[newTask setCheck:@selector(transportCountOfType:)];
	[newTask setParam1:[NSNumber numberWithInt:TRANSPORT_BARGE]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Research Steam Power"];
	[newTask setOveriview:@"Visit the Invention screen and do some research into steam."];
	[newTask setCheck:@selector(tech:)];
	[newTask setParam1:[NSNumber numberWithInt:TECH_STEAM]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Research Metal"];
	[newTask setOveriview:@"Visit the Invention screen and do some research into metal."];
	[newTask setCheck:@selector(tech:)];
	[newTask setParam1:[NSNumber numberWithInt:TECH_METAL]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Invent Rail Technolgy"];
	[newTask setOveriview:@"Visit the Invention screen and invent the railway."];
	[newTask setCheck:@selector(invented:)];
	[newTask setParam1:@"Rail"];
	[newTask setValue:[NSNumber numberWithBool:YES]];
	[self addTask:newTask];
	
	newTask = [[[Task alloc] init] retain];
	[newTask setTitle:@"Buy Train"];
	[newTask setOveriview:@"Visit the shop screen to purchase a Train to move your completed units to the city"];
	[newTask setCheck:@selector(transportCountOfType:)];
	[newTask setParam1:[NSNumber numberWithInt:TRANSPORT_TRAIN]];
	[newTask setValue:[NSNumber numberWithInt:1]];
	[self addTask:newTask];	
	 */
}

-(void)addTask:(Task*)newTask
{
	[tasksArray addObject:newTask];
}

-(int)taskCount
{
	return [tasksArray count];
}

-(Task*)getTaskAtIndex:(int)index
{
	return [tasksArray objectAtIndex:index];
}

-(void)setCurrentTask:(int)index
{
	if(index < [self taskCount])
	{
		currentTask = [self getTaskAtIndex:index];
	}
	else
	{
		currentTask = nil;
	}
}

-(Task*)currentTask
{
	return currentTask;
}

-(BOOL)getNextTask
{
	Task* thisTask = nil;
	int loop;
	
	for(loop = 0; loop < [tasksArray count]; loop++)
	{
		thisTask = [self getTaskAtIndex:loop];
		
		if(thisTask == currentTask && loop + 1 < [self taskCount])
		{
			[self setCurrentTask:loop + 1];
			return NO;
		}
	}
	
	// all tasks completed
	[self setCurrentTask:nil];
	return YES;
}

-(int)getCurrentTaskIndex
{
	Task* thisTask = nil;
	int loop;
	
	for(loop = 0; loop < [tasksArray count]; loop++)
	{
		thisTask = [self getTaskAtIndex:loop];
		
		if(thisTask == currentTask)
		{
			return loop + 1;
		}
	}

	return 0;
}

- (void)addDistance:(Distance*)newDistance
{
	[distancesArray addObject:newDistance];
}

- (Distance*)getDistanceAtIndex:(int)index
{
	return [distancesArray objectAtIndex:index];
}

- (int)distanceCount
{
	return [distancesArray count];
}

- (void)addCity:(City*)newCity
{
	[citiesArray addObject:newCity];
}

- (City*)getCityAtIndex:(int)index
{
	return [citiesArray objectAtIndex:index];
}

- (int)cityCount
{
	return [citiesArray count];
}

// Do all the end of turn processing
- (void)endTurn
{
	int loop;
	Factory* thisFactory;
	Transport* thisTransport;
	Invention* thisInvention;
	Distance* thisDistance;
	
	int stockProduced = 0; // stock produced this turn
	
	// remove all messages
	[messageArray removeAllObjects];
	
	// check for disasters
	[self generateDisaster];
	
	// decrease the player's money based on factory output - this is just a temp process
	for(loop = 0; loop < [self factoryCount]; loop++)
	{
		thisFactory = [self getFactoryAtIndex:loop];
		
		if([thisFactory owner] == currentPlayer)
		{
			if([thisFactory turnsToBuild] > 0)
			{
				if([currentPlayer canAfford:100])
				{
					[thisFactory changeTurnsToBuild:-1];
					[currentPlayer changeMoney:-100];
					
					if([thisFactory turnsToBuild] == 0)
					{
						[self addMessage:[NSString stringWithFormat:@"Construction on %@ completed.", [thisFactory name]]];
					}
					else
					{
						[self addMessage:[NSString stringWithFormat:@"Construction on %@ continues.", [thisFactory name]]];
					}	
				}
				else
				{
					[self addMessage:[NSString stringWithFormat:@"Construction halted on %@ due to insufficient funds.", [thisFactory name]]];
				}
			}
			else
			{
				if([thisFactory size] > 0)
				{
					[thisFactory changeStock:[thisFactory outputRatio] * [thisFactory workforce]];
					
					stockProduced += [thisFactory outputRatio] * [thisFactory workforce];
					
					[currentPlayer changeMoney:-[thisFactory workforce] * [thisFactory runningCost]];
					
					// increase the workforce
					[thisFactory changeWorkforce:1];
				}
			}
		}
	}
	
	// move any transport
	for(loop = 0; loop < [currentPlayer transportCount]; loop++)
	{
		thisTransport = [currentPlayer getTransportAtIndex:loop];
		
		// check to see if ordered transport has been built
		if([thisTransport turnsToBuild] > 0)
		{
			[thisTransport changeTurnsToBuild:-1];
		}
		else
		{
			if([thisTransport destination] != nil)
			{
				[thisTransport setLocation:nil];
				
				// any disasters?
				if([self randNum:50] == 1)
				{
					switch([self randNum:3])
					{
						case 0: // crash with loss of cargo
						{
							if([thisTransport load] > 0)
							{
								[self addMessage:[NSString stringWithFormat:@"%@ Crash!\n\nA %@ travelling to %@ was involved in an accident and the entire cargo was lost.",
									[thisTransport name],
									[thisTransport name],
									[thisTransport destinationName]]];
								
								[thisTransport setLoad:0];
							}
							else
							{
								[self addMessage:[NSString stringWithFormat:@"%@ Crash!\n\nA %@ travelling to %@ was involved in an accident. Luckily the transport was not carrying any cargo at the time.",
									[thisTransport name],
									[thisTransport name],
									[thisTransport destinationName]]];
							}
						}
							break;
							
						case 1: // crash transport destroyed
						{
							[self addMessage:[NSString stringWithFormat:@"%@ Lost!\n\nA %@ travelling to %@ was involved in a serious accident. The damage was so bad that the %@ was destroyed.",
								[thisTransport name],
								[thisTransport name],
								[thisTransport destinationName],
								[thisTransport name]]];
							
							[currentPlayer removeTransport:thisTransport];
							[self setCurrentTransport:nil];
		
						}
							break;
							
						case 2: // Attacked
						{
							[self addMessage:[NSString stringWithFormat:@"%@ Attacked!\n\nA %@ travelling to %@ was attacked. The damage was so bad that it will take 2 turns to repair the %@.",
								[thisTransport name],
								[thisTransport name],
								[thisTransport destinationName],
								[thisTransport name]]];
							
							[thisTransport setTurnsToBuild:2];
							
						}
					}
				}
				else if([thisTransport changeDistanceRemaining:-[thisTransport speed]])
				{
					[thisTransport setLocation:[thisTransport destination]];
										
					// unload
					if([thisTransport load] > 0)
					{
						[currentPlayer changeMoney:[thisTransport load] * [[thisTransport location] price]];
						
						[self addMessage:[NSString stringWithFormat:@"Sold %d units of stock for %d",
							[thisTransport load],
							[thisTransport load] * [[thisTransport location] price]]];
						
						// empty the transport
						[thisTransport setLoad:0];
					}
					
					// send the transport right back 
					if([thisTransport autoReturn])
					{
						if([[thisTransport destination] isKindOfClass:[Factory class]])
						{
							[self setCurrentTransport:thisTransport];
							[self loadTransport];
						}
						
						[thisTransport setDestination:[thisTransport from]];
						
						if([[thisTransport destination] isKindOfClass:[Factory class]])
						{
							
							[thisTransport setDistanceRemaining:[self distanceBetweenFactory:[thisTransport destination]
																					 andCity:[thisTransport from]]];
							
							[thisTransport setRoute:[self distanceForFactory:[thisTransport destination]
																	 andCity:[thisTransport from]]];
						}
						else
						{
							[thisTransport setDistanceRemaining:[self distanceBetweenFactory:[thisTransport from]
																					 andCity:[thisTransport destination]]];
							
							[thisTransport setRoute:[self distanceForFactory:[thisTransport from]
																	 andCity:[thisTransport destination]]];
						}
						
					}
					else
					{
						[thisTransport setDestination:nil];
						[thisTransport setRoute:nil];
					}
				}
			}
		}
	}
	
	// any transport network construction going on?
	for(loop = 0; loop < [self distanceCount]; loop++)
	{
		thisDistance = [self getDistanceAtIndex:loop];
		
		if([[thisDistance factory] owner] == currentPlayer)
		{
			if([thisDistance build:NETWORK_ROAD])
			{
				[self addMessage:[NSString stringWithFormat:@"Road from %@ to %@ completed",
					[[thisDistance factory] name],
					[[thisDistance city] name]
					]];
			}
			
			if([[currentPlayer getInventionCalled:@"Rail"] invented])
			{
				if([thisDistance construction:NETWORK_RAIL] && ![thisDistance completed:NETWORK_RAIL])
				{
					// can the player afford the construction?
					if([currentPlayer canAfford:1000])
					{
						[currentPlayer changeMoney:-1000];
						
						// any disasters?
						if([self randNum:20] == 1)
						{
							[self addMessage:[NSString stringWithFormat:@"Disaster strikes the construction of the railway between %@ and %@.",
								[[thisDistance factory] name],
								[[thisDistance city] name]
								]];
						}
						else
						{
							if([thisDistance build:NETWORK_RAIL])
							{
								[self addMessage:[NSString stringWithFormat:@"Railway from %@ to %@ completed.",
									[[thisDistance factory] name],
									[[thisDistance city] name]
									]];
							}
							else
							{
								[self addMessage:[NSString stringWithFormat:@"Building Railway from %@ to %@.",
									[[thisDistance factory] name],
									[[thisDistance city] name]
									]];
							}
						}
					}
					else
					{
						[self addMessage:[NSString stringWithFormat:@"Insufficient funds to continue building railway from %@ to %@.",
							[[thisDistance factory] name],
							[[thisDistance city] name]
							]];
					}
				}
			}
			
			if([[currentPlayer getInventionCalled:@"Canal"] invented])
			{
				if([thisDistance construction:NETWORK_CANAL] && ![thisDistance completed:NETWORK_CANAL])
				{
					// can the player afford the construction?
					if([currentPlayer canAfford:1000])
					{
						[currentPlayer changeMoney:-1000];
						
						// any disasters?
						if([self randNum:10] == 1)
						{
							[self addMessage:[NSString stringWithFormat:@"Disaster strikes the construction of the canal between %@ and %@.",
								[[thisDistance factory] name],
								[[thisDistance city] name]
								]];
						}
						else
						{
							
							if([thisDistance build:NETWORK_CANAL])
							{
								[self addMessage:[NSString stringWithFormat:@"Canal from %@ to %@ completed",
									[[thisDistance factory] name],
									[[thisDistance city] name]
									]];
							}
							else
							{
								[self addMessage:[NSString stringWithFormat:@"Building Canal from %@ to %@.",
									[[thisDistance factory] name],
									[[thisDistance city] name]
									]];
							}
						}
					}
					else
					{
						[self addMessage:[NSString stringWithFormat:@"Insufficient funds to continue building canal from %@ to %@.",
							[[thisDistance factory] name],
							[[thisDistance city] name]
							]];
					}
					
				}
			}
		}
	}
	
	// any research going on?
	for(loop = 0; loop < TECH_COUNT; loop++)
	{
		if([currentPlayer research:loop] > 0)
		{
			if([currentPlayer changeResearch:loop by:-1])
			{
				[currentPlayer changeTech:loop by:1];
				
				[self addMessage:[NSString stringWithFormat:@"Research into %@ completed.", [self technologyName:loop]]];

			}
		}
	}
	
	// any inventing going on?
	for(loop = 0; loop < [currentPlayer inventionCount]; loop++)
	{
		thisInvention = [currentPlayer getInventionAtIndex:loop];
		
		if([thisInvention turnsToComplete] > 0 && [thisInvention invent])
		{
			if([thisInvention changeTurnsToComplete:-1])
			{
				[self addMessage:[NSString stringWithFormat:@"%@ Invented", [thisInvention name]]];
				/*
				if([[thisInvention name] isEqualToString:@"Rail"])
				{
					[self addMessage:@"Railway Invented"];
				}
				
				if([[thisInvention name] isEqualToString:@"Canal"])
				{
					[self addMessage:@"Canals Invented"];
				}
				
				if([[thisInvention name] isEqualToString:@"Loom"])
				{
					[self addMessage:@"Loom Invented"];
				}
				 */
			}
			else
			{
				[self addMessage:[NSString stringWithFormat:@"%@ research in progress", [thisInvention name]]];
			}	
		
		}
	}
	
	// check to see if tasks completed
	Task* thisTask = [self currentTask];
	
	if(thisTask)
	{
		if(![thisTask completed])
		{
			NSLog(@"%s", [[thisTask value] objCType]);
			
			if([[NSString stringWithCString:[[thisTask value] objCType]] isEqualTo:@"i"])
			{
				if([currentPlayer performSelector:[thisTask check] withObject:[[thisTask param1] intValue]]  >= [[thisTask value] intValue])
				{
					[self addMessage:[NSString stringWithFormat:@"Task %@ completed", [thisTask title]]];
					
					[thisTask setCompleted:YES];
					
					[self getNextTask];
				}
			}
			else if([[NSString stringWithCString:[[thisTask value] objCType]] isEqualTo:@"c"])
			{
				BOOL boolTest = [[thisTask value] boolValue];
				BOOL boolResult = [currentPlayer performSelector:[thisTask check] withObject:[thisTask param1]];
				if(boolResult == boolTest)
				{
					[self addMessage:[NSString stringWithFormat:@"Task %@ completed", [thisTask title]]];
					
					[thisTask setCompleted:YES];
					
					[self getNextTask];
				}
			}
		}
	}
	
	Stats* newStats = [[[Stats alloc] init] retain];
	
	[newStats setProduced:stockProduced];
	[newStats setMoneyIn:[currentPlayer profit]];
	[newStats setMoneyOut:[currentPlayer loss]];
	[newStats setMoneyTotal:[currentPlayer money]];
	
	[currentPlayer addStats:newStats];
	
	// reset the profit and loss
	[currentPlayer setProfit:0];
	[currentPlayer setLoss:0];
	
	// reset the research budget
	[currentPlayer setResearchBudget:1];
	
	[self setState:STATE_END_TURN];
}

#pragma mark -
#pragma mark Disaster Handling
- (void)generateDisaster
{
}

- (int)distanceBetweenFactory:(Factory*)factory andCity:(City*)city
{
	Distance* thisDistance = nil;
	int loop;
	
	for(loop = 0; loop < [self distanceCount]; loop++)
	{
		thisDistance = [self getDistanceAtIndex:loop];
		
		if([thisDistance factory] == factory && [thisDistance city] == city)
		{
			return [thisDistance distance:NETWORK_ROAD];
		}
	}
	
	return 0;
}

- (Distance*)distanceForFactory:(Factory*)factory andCity:(City*)city
{
	Distance* thisDistance = nil;
	int loop;
	
	for(loop = 0; loop < [self distanceCount]; loop++)
	{
		thisDistance = [self getDistanceAtIndex:loop];
		
		if([thisDistance factory] == factory && [thisDistance city] == city)
		{
			return thisDistance;
		}
	}
	
	return nil;
}

-(void)addFactory:(Factory*)newFactory
{
	int subLoop;
	City* thisCity = nil;
	Distance* newDistance = nil;
	
	// add distance between factory and cities
	int distance = 5;
	
	 for(subLoop = 0; subLoop < [self cityCount]; subLoop++)
	 {
		 thisCity = [self getCityAtIndex:subLoop];
		 
		 if(subLoop == 2)
		 {
			 distance = 7;
		 }
		 else if(subLoop == 3)
		 {
			 distance = 10;
		 }
		 
		 newDistance = [[[Distance alloc] init] retain];
		 
		 [newDistance setDistance:distance forType:NETWORK_ROAD];
		 [newDistance setDistance:distance forType:NETWORK_RAIL];
		 [newDistance setDistance:distance forType:NETWORK_CANAL];
		 
		 
		[newDistance setConstruction:NO
							 forType:NETWORK_ROAD];
		[newDistance setBuilt:distance
					  forType:NETWORK_ROAD];
		
		[newDistance setCompleted:YES
						  forType:NETWORK_ROAD];
		 
		 [newDistance setFactory:newFactory];
		 [newDistance setCity:thisCity];
		 
		 [self addDistance:newDistance];
	 }
	 
	
	/*
	thisCity = [game getCityAtIndex:[self factoryCount]];
	
	newDistance = [[[Distance alloc] init] retain];
	
	[newDistance setFactory:newFactory];
	[newDistance setCity:thisCity];
	[newDistance setDistance:10 forType:NETWORK_ROAD];
	
	[game addDistance:newDistance];
	 */
	
	[factoriesArray addObject:newFactory];
}

-(int)factoryCount
{
	return [factoriesArray count];
}

-(Factory*)getFactoryAtIndex:(int)index
{
	return [factoriesArray objectAtIndex:index];
}

-(void)addMessage:(NSString*)newMessage
{
	[messageArray addObject:newMessage];
}

-(int)messageCount
{
	return [messageArray count];
}

-(NSString*)getMessageAtIndex:(int)index
{
	return [messageArray objectAtIndex:index];
}


- (BOOL)canPlayer:(Player*)player invent:(Invention*)invention
{
	int loop;
	
	// check to see if not already invented
	if([invention invent] || [invention invented])
	{
		return NO;
	}
				
	for(loop = 0; loop < TECH_COUNT; loop++)
	{
		if([player tech:loop] < [invention requiresTech:loop])
		{
			return NO;
		}
	}
	
	for(loop = 0; loop < [invention requirementCount]; loop++)
	{
		if(![player invented:[invention getRequirementAtIndex:loop]])
		{
			return NO;
		}
	}
			
	
	return YES;
}

- (NSString*)technologyName:(int)tech
{
	switch(tech)
	{
		case TECH_CONSTRUCTION: return @"Construction"; break;
		case TECH_STEAM: return @"Steam"; break;
		case TECH_PUMP: return @"Pump"; break;
		case TECH_ART: return @"Art"; break;
		case TECH_METAL: return @"Metal"; break;
		default: return @"Unknown"; break;
	}
}


#pragma mark -
#pragma mark Shop Handling

- (int)currentShopItem
{
	return currentShopItem;
}

- (int)getShopItemCount
{
    return [shopArray count];
}

- (ShopItem*)getShopItemAt:(int)index
{
    ShopItem* thisItem;
	
    thisItem = [shopArray objectAtIndex:index];
	
    return thisItem;
}

- (ShopItem*)getCurrentShopItem
{
	return [self getShopItemAt:[self currentShopItem]];
}

- (void)setCurrentShopItem:(int)newItem
{
    currentShopItem = newItem;
}

- (void)changeCurrentShopItem:(int)change
{
    currentShopItem += change;
	
    if(currentShopItem < 0)
    {
        [self setCurrentShopItem:[self getShopItemCount] - 1];
    }
	
    if(currentShopItem == [self getShopItemCount])
    {
        [self setCurrentShopItem:0];
    }
}


- (void)stockShop
{
	// create the shop array
    if(shopArray)
    {
        [shopArray release];
    }
	
    shopArray = [[NSMutableArray arrayWithCapacity:0] retain];
	
	// add items to the shop
	// stock the shop
    ShopItem* newItem;
    newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Horse & Cart"];
	[newItem setDetails:@"Basic method of transport for moving your stock to market"];
	[newItem setCost:100];
	[newItem setTurns:1];
	[newItem setProcess:@selector(buyTransport:)];
	[newItem setParam1:[NSNumber numberWithInt:TRANSPORT_CART]];
	[shopArray addObject:newItem];
	
	newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Train"];
	[newItem setDetails:@"Basic train for moving your stock to market via the rail network."];
	[newItem setCost:1000];
	[newItem setTurns:3];
	[newItem setProcess:@selector(buyTransport:)];
	[newItem setParam1:[NSNumber numberWithInt:TRANSPORT_TRAIN]];
	[shopArray addObject:newItem];
	
	newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Barge"];
	[newItem setDetails:@"Basic bargefor moving your stock to market via the cannals."];
	[newItem setCost:500];
	[newItem setTurns:2];
	[newItem setProcess:@selector(buyTransport:)];
	[newItem setParam1:[NSNumber numberWithInt:TRANSPORT_BARGE]];
	[shopArray addObject:newItem];
	
	// we will not be allowing building of new factories in this version
	/*
	newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Cotton Mill"];
	[newItem setDetails:@"A basic cotton production mill."];
	[newItem setCost:10000];
	[newItem setTurns:5];
	[newItem setProcess:@selector(buyFactory:)];
	[newItem setParam1:[NSNumber numberWithInt:FACTORY_COTTON]];
	[shopArray addObject:newItem];

	newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Pottery"];
	[newItem setDetails:@"A basic pottery."];
	[newItem setCost:10000];
	[newItem setTurns:5];
	[newItem setProcess:@selector(buyFactory:)];
	[newItem setParam1:[NSNumber numberWithInt:FACTORY_POTTERY]];
	[shopArray addObject:newItem];
	
	newItem = [[[ShopItem alloc] init] retain];
	[newItem setTitle:@"Tin Mine"];
	[newItem setDetails:@"A basic tin mine."];
	[newItem setCost:10000];
	[newItem setTurns:5];
	[newItem setProcess:@selector(buyFactory:)];
	[newItem setParam1:[NSNumber numberWithInt:FACTORY_TIN]];
	[shopArray addObject:newItem];
	*/
}

- (void)buyItem
{
	ShopItem* thisItem = [self getCurrentShopItem];
	
	[self performSelector:[thisItem process] withObject:[[thisItem param1] intValue]];
}

- (void)buyTransport:(int)type
{
	int loop;
	Distance* thisDistance;
	
	// check to see if there is a network leaving the city for the transport to run on

	BOOL built = NO;
	
	for(loop = 0; loop < [self distanceCount]; loop++)
	{
		thisDistance = [self getDistanceAtIndex:loop];
		
		if([thisDistance factory] == currentFactory && [thisDistance built:type])
		{
			built = YES;
		}
	}
	
	if(built == NO)
	{
		return;
	}
		
	Transport* newTransport = [[[Transport alloc] init] retain];
	
	[newTransport setType:type];
	
	if([[self currentPlayer] invented:[newTransport invention]]
	   && [[self currentPlayer] canAfford:[newTransport cost]])
	{
		[[self currentPlayer] addTransport:newTransport];
		[[self currentPlayer] changeMoney:-[newTransport cost]];
		[newTransport setLocation:[self currentFactory]];
	}
	else
	{
		[newTransport release];
	}
	
}

- (void)buyFactory:(int)type
{
	ShopItem* thisItem = [self getCurrentShopItem];
	
	if([[self currentPlayer] canAfford:[thisItem cost]])
	{
		Factory* newFactory = [[[Factory alloc] init] retain];
								
		[newFactory setType:type];
		[newFactory setOwner:[self currentPlayer]];
		[newFactory setTurnsToBuild:[thisItem turns]];
		[self addFactory:newFactory];
		[[self currentPlayer] changeMoney:-[thisItem cost]];
	}
}

#pragma mark -
#pragma mark XML Handling
- (void)loadXML
{
	//NSString* shopData = [NSString stringWithContentsOfFile:[@"~/test.c" stringByExpandingTildeInPath]];
	
	/*
	if([shopArray writeToFile:[@"~/IR shop.xml" stringByExpandingTildeInPath] atomically:NO] == NO)
	{
		NSLog(@"Failed to write out XML file");
	}
	 */
	
	
	
	//NSLog(@"%@", shopData);
}

#pragma mark -
#pragma mark Invention Handling
- (void)setCurrentInvention:(int)newItem
{
	currentInvention = newItem;
}

- (void)changeCurrentInvention:(int)change
{
	currentInvention += change;
	
	if(currentInvention < 0)
    {
		currentInvention = [currentPlayer inventionCount] - 1;
    }
	
    if(currentInvention == [currentPlayer inventionCount])
    {
        [self setCurrentInvention:0];
    }
	
}

- (int)currentInvention
{
	return currentInvention;
}

- (void)research:(int)type
{
	NSLog(@"Research: %@", [self technologyName:type]);
	
	if([currentPlayer researchBudget] > 0 && [currentPlayer research:type] == 0)
	{
		[currentPlayer setResearch:type
							 level:1];
		
		[currentPlayer changeResearchBudget:-1];
		
	}
}

- (void)invent
{
	int loop;
	Invention* thisInvention = [currentPlayer getInventionAtIndex:currentInvention];
	BOOL canInvent = YES;
	
	canInvent = [self canPlayer:currentPlayer
						 invent:thisInvention];
	if(canInvent)
	{
		[thisInvention setInvent:TRUE];
		
		// deduct the technology points
		
		for(loop = 0; loop < TECH_COUNT; loop++)
		{
			[currentPlayer changeTech:loop
								   by:-[thisInvention requiresTech:loop]];
		}
	}
}


- (void)populateInventionTree:(Player*)player
{
	// add some inventions
	Invention* newInvention = [[[Invention alloc] init] retain];
	[newInvention setName:@"Rail"];
	[newInvention setDetails:@"The developent of basic railway technology will allow the construction of railway tracks for running steam trains along."];
	[newInvention setTurnsToComplete:5];
	[newInvention setRequiresTech:TECH_STEAM level:1];
	[newInvention setRequiresTech:TECH_METAL level:1];
	[newInvention addRequirement:@"Steam Engine"];
	[newInvention addRequirement:@"Canal"];
	[player addInvention:newInvention];
	
	newInvention = [[[Invention alloc] init] retain];
	[newInvention setName:@"Canal"];
	[newInvention setDetails:@"This will allow the construction of the lock gates necessary to allow barges to travel up and down inclines."];
	[newInvention setTurnsToComplete:3];
	[newInvention setRequiresTech:TECH_CONSTRUCTION level:1];
	[player addInvention:newInvention];
	
	newInvention = [[[Invention alloc] init] retain];
	[newInvention setName:@"Loom"];
	[newInvention setDetails:@"The loom will allow greater productivity in the Cotton Mill."];
	[newInvention setTurnsToComplete:3];
	[newInvention setRequiresTech:TECH_ART level:1];
	[player addInvention:newInvention];
	
	// need to be set up correctly
	newInvention = [[[Invention alloc] init] retain];
	[newInvention setName:@"Steam Engine"];
	[newInvention setDetails:@"The steam engine allows pumps and other machinary to be powered. It also forms the basis for trains."];
	[newInvention setTurnsToComplete:3];
	[newInvention setRequiresTech:TECH_STEAM level:1];
	[newInvention addRequirement:@"Pump"];
	[player addInvention:newInvention];
	
	newInvention = [[[Invention alloc] init] retain];
	[newInvention setName:@"Pump"];
	[newInvention setDetails:@"The pump is used to remove water from unwanted places."];
	[newInvention setTurnsToComplete:3];
	[newInvention setRequiresTech:TECH_PUMP level:1];
	[player addInvention:newInvention];
	
}

#pragma mark -
#pragma mark Transport Handling
- (void)setCurrentTransport:(Transport*)newTransport
{
	currentTransport = newTransport;
}

- (Transport*)currentTransport
{
	return currentTransport;
}

- (void)loadTransport
{
	BOOL loaded = NO;
	
	if([currentTransport location] == currentFactory)
	{
		// keep loading until we run out of stock or transport is fully loaded
		while([currentFactory stock] > 0 && loaded == NO)
		{
			if([currentTransport changeLoad:1])
			{
				[currentFactory changeStock:-1];
			}
			else
			{
				loaded = YES;
			}
		}
	}
}

- (void)unloadTransport
{
	if([currentTransport location] == currentFactory)
	{
		if([currentTransport load] > 0)
		{
			if([currentTransport changeLoad:-1])
			{
				[currentFactory changeStock:1];
			}
		}
	}
}

- (void)autoReturnTransport:(BOOL)flag
{
	if(currentTransport)
	{
		[currentTransport setAutoReturn:flag];
	}
}

#pragma mark -
#pragma mark Factory Handling
- (void)setCurrentFactory:(Factory*)newFactory
{
	currentFactory = newFactory;
}

- (Factory*)currentFactory
{
	return currentFactory;
}

- (void)upgradeFactory
{
	if([currentFactory turnsToBuild] == 0)
	{
		[currentFactory changeSize:1];
		[currentFactory setWorkforceMax:[currentFactory workforceMax] * 2];
		[currentFactory setStockMax:[currentFactory stockMax] * 2];
		[currentFactory setOutputRatio:[currentFactory size]];
		[currentFactory setTurnsToBuild:3];
	}
}

- (void)buildFactory:(int)type
{
	if(currentFactory)
	{
		[currentFactory setOwner:[self currentPlayer]];
		[currentFactory setSize:1];
		[currentFactory setType:type];
		[currentFactory setTurnsToBuild:3];
	}
}

#pragma mark -
#pragma mark City Handling
- (void)setCurrentCity:(City*)newCity
{
	currentCity = newCity;
}

- (City*)currentCity
{
	return currentCity;
}


#pragma mark -
#pragma mark Transport Network Handling
- (Distance*)connectionBetweenCity:(City*)city factory:(Factory*)factory
{
	Distance* thisDistance;
	int loop;
	
	for(loop = 0; loop < [self distanceCount]; loop++)
	{
		thisDistance = [self getDistanceAtIndex:loop];
								
		if([thisDistance city] == city && [thisDistance factory] == factory)
		{
			return thisDistance;
		}
		
	}
	
	return nil;
}

- (void)commenceConstruction:(int)type
{
	[[self connectionBetweenCity:[self currentCity]
						 factory:[self currentFactory]] setConstruction:YES forType:type];
}

#pragma mark -
#pragma mark Help Handling
-(void)addHelp:(Help*)newHelp
{
	[helpArray addObject:newHelp];
}

-(int)helpCount
{
	return [helpArray count];
}

-(Help*)getHelpAtIndex:(int)index
{
	return [helpArray objectAtIndex:index];
}

-(void)setCurrentHelp:(int)index
{
	currentHelp = index;
}

-(int)currentHelp
{
	return currentHelp;
}

-(void)changeCurrentHelp:(int)change
{
	currentHelp += change;
	
	if(currentHelp < 0)
    {
		currentHelp = [self helpCount] - 1;
    }
	
    if(currentHelp == [self helpCount])
    {
        [self setCurrentHelp:0];
    }
}

- (void)populateHelp
{
	Help* newHelp = nil;
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Building a Factory"];
	[newHelp setContent:@"1. Select a vacant site.\n2. Click on the icon to build the factory of your choice.\n\nThe factory will be built over the next 3 turns at a cost of 100 per turn."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Current Factory Details Icons"];
	[newHelp setContent:@"The icons displayed in the top left of the screen for the current factory represent the following:\n\nSize of factory\nWorkers in factory / max number of workers allowed in the factory\nStock in factory / max amount of stock the factory can hold\nProductivity per turn per worker\nCost per worker per turn to run the factory."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Loading Transport"];
	[newHelp setContent:@"1. Select a factory.\n2. Select an item of transport and a new set of icons will appear.\n3. Select the Load icon. The transport will be automatically loaded to its maximum capacity subject to stock levels at the factory.\n\nTo unload a single item from the transport, select the Unload icon."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Moving Transport"];
	[newHelp setContent:@"1. Select a factory.\n2. Drag the required item of transport to the city you wish to move it to.\n\nTrains can only be moved to cities that have a railway connecting to them.\n\nBarges can only be moved to cities that have a canal connecting to them."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Building a Railway"];
	[newHelp setContent:@"1. Research Steam and Metal technology from the Invention screen.\n2. Research Railway\n3. Select a factory.\n4. Select the Build Railway icon.\n5. Drag the Build Railway to the city you wish to build the railway to.\nThe railway will then be built over several turns at a cost of 1000 per turn. If there is insufficient funds then construction is halted."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Building a Canal"];
	[newHelp setContent:@"1. Research Construction technology from the Invention screen.\n2. Research Canal\n3. Select a factory.\n4. Select the Build Canal icon.\n5. Drag the Build Canal to the city you wish to build the railway to.\nThe canal will then be built over several turns at a cost of 1000 per turn. If there is insufficient funds then construction is halted."];
	[self addHelp:newHelp];
	
	newHelp = [[[Help alloc] init] retain];
	[newHelp setSubject:@"Upgrading a Factory"];
	[newHelp setContent:@"1. Invent the Loom.\n2. Select a factory to upgrade.\n3. Click on the icon to upgrade the factory of your choice.\n\nThe factory will be upgraded over the next 3 turns at a cost of 100 per turn.\nDuring the upgrade no units are produced."];
	[self addHelp:newHelp];
}

// call the uDevGames 2004 vote page
- (void)vote
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.udevgames.com"]];
}

- (void) playSound:(int)sound
{
    /*
    if(![self isSoundOn])
    {
        return;
    }
	 */
    
    
	[[[soundPuck copy] autorelease] play];}

@end
