/* IRGameView */
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import <Cocoa/Cocoa.h>

@class IRController, IRGame;
@class Button;
@class Invention;
@class Factory;
@class City;
@class Transport;

enum { BUTTON_NONE,
	BUTTON_START_GAME,
	BUTTON_GO_OFFICE, BUTTON_GO_STATS, BUTTON_GO_MAP, BUTTON_GO_INVENTIONS, BUTTON_GO_SHOP, BUTTON_GO_HELP,
	BUTTON_BUY_FACTORY, BUTTON_BUY_FACTORY_COTTON, BUTTON_BUY_FACTORY_POTTERY, BUTTON_BUY_FACTORY_TIN,
	BUTTON_BUY_TRANSPORT, BUTTON_BUY_TRANSPORT_CART, BUTTON_BUY_TRANSPORT_TRAIN, BUTTON_BUY_TRANSPORT_BARGE,
	BUTTON_END_TURN,
	BUTTON_CANCEL, BUTTON_CONTINUE,
	BUTTON_TRANSPORT_LOAD_INC, BUTTON_TRANSPORT_LOAD_DEC, BUTTON_TRANSPORT_SEND_TO_CITY, BUTTON_TRANSPORT_SEND_TO_FACTORY, BUTTON_TRANSPORT_MOVE_TO,
	BUTTON_INVENT, BUTTON_RESEARCH,
	BUTTON_NEXT, BUTTON_PREVIOUS, BUTTON_BUY,
	BUTTON_QUIT,
	BUTTONS_LAST };

@interface IRGameView : NSView
{
    IBOutlet IRController *controller;
    IBOutlet IRGame *game;
	
	int		buttons[BUTTONS_LAST];	// tracking number for each tracking rectangle
	
	NSMutableArray* arrayButtons;
	
	int		currentButton;	// currently selected button item
	NSPoint	lastMousePoint;		// last known mouse point used for tracking rects
	BOOL	mouseUp;		// is mouse button up
	
	Button* buttonCurrent;
	SEL displayMethod;		// which display routine to use
	
	NSRect windowRect;
	
	int currentShopItem;	// the current shop item being displayed
	
	NSPoint selectionPoint;		// point on screen to display selection list at
	NSMutableArray* selectionItemsArray; // items in the selection window
	int currentSelection;		// the currently selected item in the selection list
	
	BOOL seekingDestination;	// are we seeking a transport destination?
	
	NSImage* imageWoodPanel;
	NSImage* imageAboveWoodPanel;
	NSImage* imageGrassTexture;
	
	id dragged;					// the current item that is being dragged
	NSPoint mousePoint;			// current mouse position
	
	NSMutableArray* iconWheelItemsArray;	// array of items in the icon wheel
	id iconWheelParent;			// partent object for the icon wheel
}
- (SEL)displayMethod;
- (void)setDisplayState;
// displays
- (void)displayTitle;
- (void)displayOffice;
- (void)displayStats;
- (void)displayMap;
- (void)displayShop;
- (void)displayEndTurn;
- (void)displayInventions;
- (void)displayHelp;
- (void)displayCredits;
- (void)displayUnimplemented;

- (void)drawCurrentTask;

#pragma mark -
#pragma mark Icon Wheel Handling
- (void)addIconWheelItem:(id)newItem;
- (int)iconWheelCount;
- (id)getIconWheelItemAtIndex:(int)index;
- (void)resetIconWheel;

#pragma mark -
#pragma mark Selection List Handling
- (void)addSelectionItem:(id)newItem;
- (int)selectionItemsCount;
- (id)getSelectionItemAtIndex:(int)index;
- (void)resetSelectionItems;

// utilities methods
- (void)displayText:(NSString*)text atPoint:(NSPoint)point fontSize:(int)fontSize colour:(NSColor*)colour;
- (float)displayText:(NSString*)text atPoint:(NSPoint)point width:(int)width height:(int)height fontSize:(int)fontSize colour:(NSColor*)colour;

- (void)setTrackingAreas;
- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)newMethod;
- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)method imageOn:(NSImage*)imageOn imageOff:(NSImage*)imageOff;
- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)method data:(id)data;
- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)method data:(id)data imageOn:(NSImage*)imageOn imageOff:(NSImage*)imageOff;

- (id)getButtonWithID:(int)buttonID;

- (void)buttonChanged:(NSNotification*)notification;

- (void)buildStandardButtons;

- (void)loadGraphics;

- (void)setDragged:(id)newDrag;
- (id)dragged;

- (void)buildIconWheel:(id)item;

@end
