//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import "IRGameView.h"
#import "IRGame.h"
#import "IRController.h"

#import "Button.h"
#import "Player.h"
#import "Factory.h"
#import "Transport.h"
#import "City.h"
#import "Distance.h"
#import "Invention.h"
#import "Task.h"
#import "ShopItem.h"
#import "Action.h"
#import "Stats.h"
#import "Help.h"

@implementation IRGameView

- (id)initWithFrame:(NSRect)frameRect
{
	if ((self = [super initWithFrame:frameRect]) != nil) {
		// Add initialization code here
		
		arrayButtons = [[NSMutableArray arrayWithCapacity:0] retain];
		
		selectionItemsArray = [[NSMutableArray arrayWithCapacity:0] retain];
		iconWheelItemsArray = [[NSMutableArray arrayWithCapacity:0] retain];
		
		[self loadGraphics];
		
		buttonCurrent = nil;
		
		[self setDragged:nil];
		
		
		seekingDestination = NO;
			
		mouseUp = YES;
		
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(buttonChanged:)
													 name:@"IRButtonChanged"
												   object:nil];
	}
	return self;
}

- (BOOL)acceptsFirstResponder
{
	[[self window] makeFirstResponder:self];
    [[self window] setAcceptsMouseMovedEvents:YES];
    return YES;
}

- (SEL)displayMethod
{
	return displayMethod;
}

- (void)setDisplayState
{
	switch([game state])
	{
		case STATE_TITLE:
			displayMethod = @selector(displayTitle);
			break;
			
		case STATE_OFFICE:
			displayMethod = @selector(displayOffice);
			break;
			
		case STATE_STATS:
			displayMethod = @selector(displayStats);
			break;
			
		case STATE_MAP:
			displayMethod = @selector(displayMap);
			break;
			
		case STATE_SHOP:
			displayMethod = @selector(displayShop);
			break;
			
		case STATE_INVENTIONS:
			displayMethod = @selector(displayInventions);
			break;
			
		case STATE_END_TURN:
			displayMethod = @selector(displayEndTurn);
			break;
			
		case STATE_HELP:
			displayMethod = @selector(displayHelp);
			break;
			
		case STATE_CREDITS:
			displayMethod = @selector(displayCredits);
			break;
			
		default:
			displayMethod = @selector(displayUnimplemented);
			break;
			
	}	
}

- (void)drawRect:(NSRect)rect
{
	int loop;

	windowRect = rect;
			
	[self performSelector:displayMethod];
	
	// display the Icon Wheel
	float wheelRadius;
	
	if([self selectionItemsCount] > 0)
	{
		NSBezierPath *pathWheel = [NSBezierPath bezierPath]; // path for the Icon Wheel background
		NSBezierPath *pathIcons = [NSBezierPath bezierPath];
		
		[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.4 alpha:1.0] colorWithAlphaComponent:0.9] set];
		
		if([self selectionItemsCount] <= 6)
		{
			wheelRadius = 90;
		}
		else if([self selectionItemsCount] > 6 && [self selectionItemsCount] <= 10)
		{
			wheelRadius = 120;
		}
		else
		{
			wheelRadius = 150;
		}
		
		[pathWheel appendBezierPathWithOvalInRect:NSMakeRect(selectionPoint.x - wheelRadius,
															 selectionPoint.y - wheelRadius,
															 wheelRadius * 2, wheelRadius * 2)];
		[pathWheel fill];
		[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.9 alpha:1.0] colorWithAlphaComponent:0.8] set];
		
		[pathWheel stroke];
		
		float iconX, iconY;
				
		float angle = 360 / ([self selectionItemsCount]);
		for(loop = 0; loop < [self selectionItemsCount]; loop++)
		{
			[pathIcons appendBezierPathWithArcWithCenter:selectionPoint
												  radius:wheelRadius - 30
											  startAngle:(angle * loop) + 90
												endAngle:((angle + 1) * loop) + 90
											   clockwise:YES];
			
			iconX = [pathIcons currentPoint].x - 24;
			iconY = [pathIcons currentPoint].y - 24;
			
			[[self getSelectionItemAtIndex:loop] setPositionIconWheel:NSMakePoint(iconX, iconY)];
		
			
		}
		
		for(loop = 0; loop < [self selectionItemsCount]; loop++)
		{
			if(NSPointInRect(mousePoint, NSMakeRect([[self getSelectionItemAtIndex:loop] positionIconWheel].x, [[self getSelectionItemAtIndex:loop] positionIconWheel].y, 48, 48)))
			{
				[[[self getSelectionItemAtIndex:loop] image] compositeToPoint:[[self getSelectionItemAtIndex:loop] positionIconWheel]
																	operation:NSCompositeSourceOver
																	 fraction:0.9];
			}
			else
			{
				[[[self getSelectionItemAtIndex:loop] image] compositeToPoint:[[self getSelectionItemAtIndex:loop] positionIconWheel]
																	operation:NSCompositeSourceOver
																	 fraction:0.5];
				
			}
			
		}
		
		for(loop = 0; loop < [self selectionItemsCount]; loop++)
		{
			if(NSPointInRect(mousePoint, NSMakeRect([[self getSelectionItemAtIndex:loop] positionIconWheel].x, [[self getSelectionItemAtIndex:loop] positionIconWheel].y, 48, 48)))
			{
				[self displayText:[NSString stringWithFormat:@"%@",
					[[self getSelectionItemAtIndex:loop] name]]
						  atPoint:NSMakePoint(selectionPoint.x - wheelRadius + (wheelRadius / 2), selectionPoint.y)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				if([[self getSelectionItemAtIndex:loop] isKindOfClass:[Transport class]])
				{
					Transport* thisTransport = [self getSelectionItemAtIndex:loop];
					
					if([thisTransport turnsToBuild] > 0)
					{
						[self displayText:[NSString stringWithFormat:@"%d turn%s until ready",
							[thisTransport turnsToBuild],
							[thisTransport turnsToBuild] == 1 ? "" : "s"]
								  atPoint:NSMakePoint(selectionPoint.x - wheelRadius + (wheelRadius / 2), selectionPoint.y - 15)
								 fontSize:12
								   colour:[NSColor whiteColor]];
					}
					else
					{
						[self displayText:[NSString stringWithFormat:@"Load %d",
							[thisTransport load]]
								  atPoint:NSMakePoint(selectionPoint.x - wheelRadius + (wheelRadius / 2), selectionPoint.y - 15)
								 fontSize:12
								   colour:[NSColor whiteColor]];
					}
				}
			}
		}
	}
	
	// display anything that is being dragged
	if([self dragged])
	{
		[[[self dragged] image] compositeToPoint:NSMakePoint(mousePoint.x - 24, mousePoint.y - 24)
									   operation:NSCompositeSourceOver
										fraction:0.5];
	
		// draw a line from point of drag
		NSBezierPath *pathDragLine = [NSBezierPath bezierPath];
		
		[pathDragLine moveToPoint:NSMakePoint([[self dragged] positionIconWheel].x + 24, [[self dragged] positionIconWheel].y + 24)];
		[pathDragLine lineToPoint:mousePoint];
		
		[[NSColor whiteColor] set];

		[pathDragLine stroke];
	}
	
	
	
	// draw buttons
	Button* button;
	for(loop = 0; loop < [arrayButtons count]; loop++)
	{
		button = [arrayButtons objectAtIndex:loop];
		
		[button displayButton];
	}

	// display tool tip for currently selected button
	if(buttonCurrent)
	{
		if([buttonCurrent inside] && [buttonCurrent imageOn] != nil)
		{
			NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
			
			[attr setObject:[NSFont fontWithName:@"Geneva" size:10]
					 forKey:NSFontAttributeName];
			
			[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.4 alpha:1.0] colorWithAlphaComponent:0.8] set];
			
			NSRectFillUsingOperation(NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, [[buttonCurrent text] sizeWithAttributes:attr].width, 12), NSCompositeSourceOver);

			[self displayText:[buttonCurrent text]
					  atPoint:NSMakePoint([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y)
					 fontSize:10
					   colour:[NSColor whiteColor]];
		}
	}
	
	/*
	// display selection menu
	for(loop = 0; loop < [self selectionItemsCount]; loop++)
	{
		// draw background box
		if(loop == currentSelection)
		{
			[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.4 alpha:1.0] colorWithAlphaComponent:0.8] set];
		}
		else
		{
			[[[NSColor colorWithDeviceRed:0.0 green:0.1 blue:0.1 alpha:1.0] colorWithAlphaComponent:0.8] set];
		}
		
		NSRectFillUsingOperation(NSMakeRect(selectionPoint.x, selectionPoint.y - (16 * loop), 200, 16), NSCompositeSourceOver);
		
		[self displayText:[NSString stringWithFormat:@"%@",
			[[self getSelectionItemAtIndex:loop] name]]
				  atPoint:NSMakePoint(selectionPoint.x, selectionPoint.y - (16 * loop) - 1)
				 fontSize:14
				   colour:[NSColor whiteColor]];
	}
	 */
}

- (void)buildStandardButtons
{
	[self createTrackingArea:NSMakeRect(0, 0, 48, 48)
						item:BUTTON_GO_STATS
						text:@"Go to Stats"
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_STATS
					 imageOn:[NSImage imageNamed:@"stats_on"]
					imageOff:[NSImage imageNamed:@"stats_off"]];
	
	[self createTrackingArea:NSMakeRect(50, 0, 48, 48)
						item:BUTTON_GO_INVENTIONS
						text:@"Go to Inventions"
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_INVENTIONS
					 imageOn:[NSImage imageNamed:@"invent_on"]
					imageOff:[NSImage imageNamed:@"invent_off"]
		];
	
	[self createTrackingArea:NSMakeRect(100, 0, 48, 48)
						item:BUTTON_GO_OFFICE
						text:@"Go to Office"
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_OFFICE
					 imageOn:[NSImage imageNamed:@"office_on"]
					imageOff:[NSImage imageNamed:@"office_off"]];
	
	[self createTrackingArea:NSMakeRect(150, 0, 48, 48)
						item:BUTTON_GO_MAP
						text:@"Go to Map"
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_MAP
					 imageOn:[NSImage imageNamed:@"map_on"]
					imageOff:[NSImage imageNamed:@"map_off"]];	
	
	/*
	[self createTrackingArea:NSMakeRect(200, 0, 48, 48)
						item:BUTTON_GO_SHOP
						text:@"Go to Shop"
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_SHOP
					 imageOn:[NSImage imageNamed:@"shop_on"]
					imageOff:[NSImage imageNamed:@"shop_off"]];	
	 */
	
	[self createTrackingArea:NSMakeRect(325, 0, 150, 50)
						item:BUTTON_END_TURN
						text:@""
					 enabled:YES
					  method:@selector(endTurn)
					 imageOn:[NSImage imageNamed:@"end_turn_on"]
					imageOff:[NSImage imageNamed:@"end_turn_off"]];
	
	[self createTrackingArea:NSMakeRect(500, 0, 150, 50)
						item:BUTTON_GO_HELP
						text:@""
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_HELP
					 imageOn:[NSImage imageNamed:@"help_on"]
					imageOff:[NSImage imageNamed:@"help_off"]
		];
	
	[self createTrackingArea:NSMakeRect(650, 0, 150, 50)
						item:BUTTON_QUIT
						text:@""
					 enabled:YES
					  method:@selector(setState:)
						data:STATE_TITLE
					 imageOn:[NSImage imageNamed:@"quit_on"]
					imageOff:[NSImage imageNamed:@"quit_off"]];
	
}

- (void)setTrackingAreas
{
    int loop;
    		
	buttonCurrent = nil;
	
	[arrayButtons removeAllObjects];
	
    // remove existing areas
    for(loop = 0; loop < BUTTONS_LAST; loop++)
    {
        if(buttons[loop] != -1)
        {
            [self removeTrackingRect:buttons[loop]];
        }
    }
		
    currentButton = BUTTON_NONE;
	
	
    switch([game state])
    {
#pragma mark -
#pragma mark Tracking:Title
        case STATE_TITLE:
			
            [self createTrackingArea:NSMakeRect(10, 0, 150, 50)
								item:BUTTON_START_GAME
								text:@""
							 enabled:YES
							  method:@selector(newGame:)
								data:NO
							 imageOn:[NSImage imageNamed:@"start_game_on"]
							imageOff:[NSImage imageNamed:@"start_game_off"]];
			
			[self createTrackingArea:NSMakeRect(160, 0, 150, 50)
								item:BUTTON_START_GAME
								text:@""
							 enabled:YES
							  method:@selector(newQuickStartGame)
							 imageOn:[NSImage imageNamed:@"quick_start_game_on"]
							imageOff:[NSImage imageNamed:@"quick_start_game_off"]];
			
			[self createTrackingArea:NSMakeRect(310, 0, 150, 50)
								item:BUTTON_START_GAME
								text:@""
							 enabled:YES
							  method:@selector(newGame:)
								data:YES
							 imageOn:[NSImage imageNamed:@"tutorial_game_on"]
							imageOff:[NSImage imageNamed:@"tutorial_game_off"]];
			
			/*
			[self createTrackingArea:NSMakeRect(490, 0, 150, 50)
								item:BUTTON_START_GAME
								text:@""
							 enabled:YES
							  method:@selector(setState:)
								data:STATE_CREDITS
							 imageOn:[NSImage imageNamed:@"credits_on"]
							imageOff:[NSImage imageNamed:@"credits_off"]];
			 */
				
			[self createTrackingArea:NSMakeRect(640, 0, 150, 50)
								item:BUTTON_START_GAME
								text:@""
							 enabled:YES
							  method:@selector(vote)
								data:NO
							 imageOn:[NSImage imageNamed:@"vote_on"]
							imageOff:[NSImage imageNamed:@"vote_off"]];
			
			break;
			
#pragma mark -
#pragma mark Tracking:Credits
        case STATE_CREDITS:
			[self createTrackingArea:NSMakeRect(580, 15, 100, 25)
								item:BUTTON_START_GAME
								text:@"Title Screen"
							 enabled:YES
							  method:@selector(setState:)
								data:STATE_TITLE];
			break;
			
#pragma mark -
#pragma mark Tracking:Office
		case STATE_OFFICE:
			[self buildStandardButtons];			

			break;
			
#pragma mark -
#pragma mark Tracking:Stats
		case STATE_STATS:
			[self buildStandardButtons];			
						break;
			
#pragma mark -
#pragma mark Tracking:Map
		case STATE_MAP:
			[self buildStandardButtons];		
						break;
			
#pragma mark -
#pragma mark Tracking:Help
		case STATE_HELP:
			[self buildStandardButtons];	
			
			[self createTrackingArea:NSMakeRect(40, 175, 48, 48)
								item:BUTTON_PREVIOUS
								text:@"Previous"
							 enabled:YES
							  method:@selector(changeCurrentHelp:)
								data:-1
							 imageOn:[NSImage imageNamed:@"previous_on"]
							imageOff:[NSImage imageNamed:@"previous_off"]];
			
			[self createTrackingArea:NSMakeRect(140, 175, 48, 48)
								item:BUTTON_NEXT
								text:@"Next"
							 enabled:YES
							  method:@selector(changeCurrentHelp:)
								data:1
							 imageOn:[NSImage imageNamed:@"next_on"]
							imageOff:[NSImage imageNamed:@"next_off"]];
			break;
			
#pragma mark -
#pragma mark Tracking:Inventions
		case STATE_INVENTIONS:
			[self buildStandardButtons];			
			
			// create dynamic buttons for research
			for(loop = 0; loop < TECH_COUNT; loop++)
			{
				[self createTrackingArea:NSMakeRect(160, 450 - (loop * 12), 90, 11)
									item:BUTTON_RESEARCH
									text:@"Research"
								 enabled:YES
								  method:@selector(research:)
									data:loop];			
			}
				
				
				[self createTrackingArea:NSMakeRect(550, 440, 48, 48)
									item:BUTTON_PREVIOUS
									text:@"Previous"
								 enabled:YES
								  method:@selector(changeCurrentInvention:)
									data:-1
								 imageOn:[NSImage imageNamed:@"previous_on"]
								imageOff:[NSImage imageNamed:@"previous_off"]];
			
				[self createTrackingArea:NSMakeRect(650, 440, 48, 48)
									item:BUTTON_NEXT
									text:@"Next"
								 enabled:YES
								  method:@selector(changeCurrentInvention:)
									data:1
								 imageOn:[NSImage imageNamed:@"next_on"]
								imageOff:[NSImage imageNamed:@"next_off"]];
			
			[self createTrackingArea:NSMakeRect(600, 440, 48, 48)
								item:BUTTON_INVENT
								text:@"Invent"
							 enabled:YES
							  method:@selector(invent)
							 imageOn:[NSImage imageNamed:@"invent_on"]
							imageOff:[NSImage imageNamed:@"invent_off"]
				];
			break;
			
#pragma mark -
#pragma mark Tracking:Shop
		case STATE_SHOP:
			[self buildStandardButtons];			

			
			[self createTrackingArea:NSMakeRect(10, 140, 48, 48)
								item:BUTTON_PREVIOUS
								text:@"Previous"
							 enabled:YES
							  method:@selector(changeCurrentShopItem:)
								data:-1
							 imageOn:[NSImage imageNamed:@"previous_on"]
							imageOff:[NSImage imageNamed:@"previous_off"]
					];
			
			[self createTrackingArea:NSMakeRect(110, 140, 48, 48)
								item:BUTTON_NEXT
								text:@"Next"
							 enabled:YES
							  method:@selector(changeCurrentShopItem:)
								data:1
							 imageOn:[NSImage imageNamed:@"next_on"]
							imageOff:[NSImage imageNamed:@"next_off"]
				];
			
	
			[self createTrackingArea:NSMakeRect(60, 140, 48, 48)
								item:BUTTON_BUY
								text:@"Buy"
							 enabled:YES
							  method:@selector(buyItem)
							 imageOn:[NSImage imageNamed:@"shop_on"]
							imageOff:[NSImage imageNamed:@"shop_off"]
				];
			
			
			break;
						
			
#pragma mark -
#pragma mark Tracking:End Turn
		case STATE_END_TURN:
			[self createTrackingArea:NSMakeRect(325, 0, 150, 50)
								item:BUTTON_CONTINUE
								text:@""
							 enabled:YES
							  method:@selector(setState:)
								data:STATE_MAP
							 imageOn:[NSImage imageNamed:@"continue_on"]
							imageOff:[NSImage imageNamed:@"continue_off"]];
			break;
    }
}

- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)newMethod
{
	Button* button = [[[Button alloc] init] retain];
	
	[button setButtonID:item];
	[button setRect:rect];
	[button setText:buttonText];
	
	[button setEnabled:enabled];
	[button setData:nil];
	[button setMethod:newMethod];
	[button setNumParams:0];
	
	[arrayButtons addObject:button];
	
	
    if(NSPointInRect(mousePoint, rect))
    {
		[button setInside:YES];
		buttonCurrent = button;
    }
}

- (void)createTrackingArea:(NSRect)rect item:(int)item text:(NSString*)buttonText enabled:(BOOL)enabled method:(SEL)method data:(id)data
{	
	Button* button = [[[Button alloc] init] retain];
	
	[button setButtonID:item];
	[button setRect:rect];
	[button setText:buttonText];
	
	[button setEnabled:enabled];
	
	[button setData:data];
	[button setMethod:method];

	[button setNumParams:1];

	
	[arrayButtons addObject:button];
	
	if(NSPointInRect(mousePoint, rect))
    {
		[button setInside:YES];
		buttonCurrent = button;
    }
}

- (void)createTrackingArea:(NSRect)rect
					  item:(int)item
					  text:(NSString*)buttonText
				   enabled:(BOOL)enabled
					method:(SEL)method
				   imageOn:(NSImage*)imageOn
				  imageOff:(NSImage*)imageOff
{
	Button* button = [Button initWithID:item
							   position:rect
								   text:buttonText];
	
	[button setEnabled:enabled];
	[button setImageOn:imageOn
			  imageOff:imageOff];
	
	[button setData:nil];
	[button setMethod:method];
	[button setNumParams:0];

	
	[arrayButtons addObject:button];
	
	if(NSPointInRect(mousePoint, rect))
    {
		[button setInside:YES];
		buttonCurrent = button;
    }
}

- (void)createTrackingArea:(NSRect)rect
					  item:(int)item
					  text:(NSString*)buttonText
				   enabled:(BOOL)enabled
					method:(SEL)method
					  data:(id)data
				   imageOn:(NSImage*)imageOn
				  imageOff:(NSImage*)imageOff
{
	Button* button = [[[Button alloc] init] retain];
	
	[button setButtonID:item];
	[button setRect:rect];
	[button setText:buttonText];
	
	[button setEnabled:enabled];
	[button setImageOn:imageOn
			  imageOff:imageOff];
	
	[button setData:data];
	[button setMethod:method];
	[button setNumParams:1];

	
	[arrayButtons addObject:button];
	
	if(NSPointInRect(mousePoint, rect))
    {
		[button setInside:YES];
		buttonCurrent = button;
    }
}


- (id)getButtonWithID:(int)buttonID
{
	int loop;
	Button* button;
	
	for(loop = 0; loop < [arrayButtons count]; loop++)
	{
		button = [arrayButtons objectAtIndex:loop];
		
		if([button buttonID] == buttonID)
		{
			return button;
		}
	}
	
	return nil;
}

- (void)displayText:(NSString*)text atPoint:(NSPoint)point fontSize:(int)fontSize colour:(NSColor*)colour
{
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:colour
             forKey:NSForegroundColorAttributeName];
	
    [attr setObject:[NSFont fontWithName:@"Geneva" size:fontSize]
			 forKey:NSFontAttributeName];
	
    
	
    [text drawAtPoint:point withAttributes:attr];
	
    
}

- (float)displayText:(NSString*)text atPoint:(NSPoint)point width:(int)width height:(int)height fontSize:(int)fontSize colour:(NSColor*)colour
{
	NSMutableParagraphStyle* paraStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[paraStyle setAlignment:NSLeftTextAlignment];
	
	
    NSMutableDictionary *attr = [NSMutableDictionary dictionaryWithCapacity:0];
    [attr setObject:colour
			 forKey:NSForegroundColorAttributeName];
	
	[attr setObject:[NSFont fontWithName:@"Geneva" size:fontSize]
			 forKey:NSFontAttributeName];
	
	[attr setObject:paraStyle
			 forKey:NSParagraphStyleAttributeName];
		
    NSRect rcText = NSMakeRect(point.x, point.y - height, width, height);
	
    [text drawInRect:rcText
	  withAttributes:attr];
	
	int loop;
	int lines = (([@"A" sizeWithAttributes:attr].width * [text length]) / width);
	for(loop = 0; loop < [text length]; loop++)
	{
		if([text characterAtIndex:loop] == '\n')
		{
			lines++;
		}
	}
	
	/*
	rcText.size.height = (lines + 2) * fontSize;
	rcText.origin.y = point.y - rcText.size.height;

	
	[[[NSColor colorWithDeviceRed:0.8 green:0.4 blue:0.9 alpha:1.0] colorWithAlphaComponent:0.2] set];
	NSRectFillUsingOperation(rcText, NSCompositeSourceOver);
	
	rcText.size.height -= 4;
	rcText.size.width -= 4;
	rcText.origin.x += 2;
	rcText.origin.y += 2;
	
	[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.9 alpha:1.0] colorWithAlphaComponent:0.2] set];

	NSRectFillUsingOperation(rcText, NSCompositeSourceOver);
	 */
	
	
	return (lines + 2) * fontSize;
}

-(void)mouseDragged:(NSEvent *)theEvent
{
	NSPoint eventLocation = [theEvent locationInWindow];
	NSPoint centre = [self convertPoint:eventLocation fromView:nil];
	int loop;
	
	mousePoint = centre;
	
	if([self dragged])
	{
		[self setNeedsDisplay:YES];
	}
	else
	{
		// icon wheel code
		if([self selectionItemsCount] > 0)
		{
			for(loop = 0; loop < [self selectionItemsCount]; loop++)
			{				
				if(NSPointInRect(mousePoint, NSMakeRect([[self getSelectionItemAtIndex:loop] positionIconWheel].x,
														[[self getSelectionItemAtIndex:loop] positionIconWheel].y,
														48, 48)))
				{
					// is a transport item selected?
					if([[self getSelectionItemAtIndex:loop] isKindOfClass:[Transport class]])
					{
						Transport* thisTransport = [self getSelectionItemAtIndex:loop];
						
						if([thisTransport turnsToBuild] == 0)
						{
							[self setDragged:[self getSelectionItemAtIndex:loop]];
							[game setCurrentTransport:[self getSelectionItemAtIndex:loop]];
						}
					}
					
					// is an action item selected?
					if([[self getSelectionItemAtIndex:loop] isKindOfClass:[Action class]])
					{
						if([[self getSelectionItemAtIndex:loop] draggable])
						{
							[self setDragged:[self getSelectionItemAtIndex:loop]];
						}
					}
				}
			}
		}
	}
}

- (void)mouseMoved:(NSEvent *)theEvent
{
	NSPoint eventLocation = [theEvent locationInWindow];
	NSPoint centre = [self convertPoint:eventLocation fromView:nil];
		
	mousePoint = centre;
	
	// check to see if we are in a button or not
	int loop;
	Button* button;
	BOOL inButton = NO;
	for(loop = 0; loop < [arrayButtons count]; loop++)
	{
		button = [arrayButtons objectAtIndex:loop];
		
		if(NSPointInRect(centre, [button rect]))
		{
			inButton = YES;
			
			if(buttonCurrent != button)
			{
				if(buttonCurrent)
				{
					[buttonCurrent setInside:NO];
				
					[self setNeedsDisplayInRect:NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, 100, 15)];
					[self setNeedsDisplayInRect:[buttonCurrent rect]];
				}
				
				buttonCurrent = button;
				[button setInside:YES];
				
				[self setNeedsDisplayInRect:[buttonCurrent rect]];
				[self setNeedsDisplayInRect:NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, 100, 15)];	
				
				break;
			}
		}
	}
	
	if(inButton == NO)
	{
		[buttonCurrent setInside:NO];
		
		[self setNeedsDisplayInRect:NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, 100, 15)];
		[self setNeedsDisplayInRect:[buttonCurrent rect]];

		buttonCurrent = nil;
	}
	
	if([self selectionItemsCount] > 0)
	{
		[self setNeedsDisplay:YES];
	}
}

- (void)mouseDown:(NSEvent *)theEvent
{		
}

- (void)mouseUp:(NSEvent *)theEvent
{	
	NSPoint eventLocation = [theEvent locationInWindow];
	NSPoint centre = [self convertPoint:eventLocation fromView:nil];
	int loop;
		
	// icon wheel code
	if([self selectionItemsCount] > 0)
	{		
		for(loop = 0; loop < [self selectionItemsCount]; loop++)
		{
			NSLog(@"item selected:%@", [self getSelectionItemAtIndex:loop]);

			if(NSPointInRect(mousePoint, NSMakeRect([[self getSelectionItemAtIndex:loop] positionIconWheel].x,
													[[self getSelectionItemAtIndex:loop] positionIconWheel].y,
													48, 48)))
			{
				[game playSound:0];

				// is an item of transport selected?
				if([[self getSelectionItemAtIndex:loop] isKindOfClass:[Transport class]])
				{
					Transport* thisTransport = [self getSelectionItemAtIndex:loop];
					
					if([thisTransport turnsToBuild] == 0)
					{
						[game setCurrentTransport:thisTransport];					
						[self buildIconWheel:thisTransport];
						[self setNeedsDisplay:YES];
					}
					
					return;
				}
				
				// is an action item selected?
				if([[self getSelectionItemAtIndex:loop] isKindOfClass:[Action class]])
				{
					Action* thisAction = [self getSelectionItemAtIndex:loop];
					
					// ensure we have not clicked something designed for dragging
					if(![thisAction draggable])
					{
						NSLog(@"Selected Action: %@", [thisAction name]);
						if([thisAction param1] == nil)
						{
							[game performSelector:[thisAction process]];
							[self setNeedsDisplay:YES];
							return;
						}
						else
						{
							NSLog(@"has a param");
							[game performSelector:[thisAction process] withObject:[[thisAction param1] intValue]];
							[self buildIconWheel:nil];
							[self setNeedsDisplay:YES];
							return;
						}					
					}
				}
			}
		}
	}
		
	if(buttonCurrent)
	{
		if([buttonCurrent inside])
		{		
			[game playSound:0];
			
			if([buttonCurrent numParams] == 0)
			{
				[game performSelector:[buttonCurrent method]];
			}
			else
			{
				[game performSelector:[buttonCurrent method] withObject:[buttonCurrent data]];
			}
		}
	}
	
	
	switch([game state])
	{
		
#pragma mark -
#pragma mark Map
		case STATE_MAP:
		{
			Transport* thisTransport;
			Factory* thisFactory;
			Player* thisPlayer = [game currentPlayer];
			
			for(loop = 0; loop < [game factoryCount]; loop++)
			{
				thisFactory = [game getFactoryAtIndex:loop];
				
				if(NSPointInRect(centre, NSMakeRect([thisFactory positionMap].x - 24, [thisFactory positionMap].y - 24, 48, 48)))
				{
					NSLog(@"Factory: %@ clicked", [thisFactory name]);
					[game setCurrentFactory:thisFactory];
					
					if([self dragged])
					{
						if([game currentTransport] != nil && [[game currentTransport] location] != nil)
						{
							if([game connectionBetweenCity:[[game currentTransport] location]
												   factory:thisFactory])
							{
								[[game currentTransport] setDestination:[game currentFactory]];
								[[game currentTransport] setDistanceRemaining:[game distanceBetweenFactory:[game currentFactory]
																								   andCity:[[game currentTransport] location]]];
								
								[[game currentTransport] setRoute:[game distanceForFactory:[game currentFactory]
																				   andCity:[[game currentTransport] location]]];
								[[game currentTransport] setLocation:nil];
								
								[self setDragged:nil];
								[self resetSelectionItems];
								[self setNeedsDisplay:YES];
								return;
							}
						}								
					}
					else
					{
						selectionPoint = centre;
						currentSelection = 0;
						
						// build a new icon wheel
						[self buildIconWheel:thisFactory];
						[self setNeedsDisplay:YES];
						return;
					}
					break;
				}
			}
			
			City* thisCity;
			for(loop = 0; loop < [game cityCount]; loop++)
			{
				thisCity = [game getCityAtIndex:loop];
				
				if(NSPointInRect(centre, NSMakeRect([thisCity positionMap].x - 24, [thisCity positionMap].y - 24, 48, 48)))
				{
					NSLog(@"City: %@ clicked", [thisCity name]);
					selectionPoint = centre;
					currentSelection = 0;
					
					[game setCurrentCity:thisCity];
					
					// check to see if dragging transport to the city
					if([self dragged])
					{
						// is an action item selected?
						if([[self dragged] isKindOfClass:[Action class]])
						{
							Action* thisAction = [self dragged];
							
							NSLog(@"Selected Action: %@", [thisAction name]);
							[game performSelector:[thisAction process] withObject:[[thisAction param1] intValue]];
							
							[self setDragged:nil];
							[self resetSelectionItems];
							[self setNeedsDisplay:YES];
							return;
						}
						
						if([[self dragged] isKindOfClass:[Transport class]])
						{
							if([game currentTransport] != nil && [[game currentTransport] location] != nil)
							{
								if([[game connectionBetweenCity:thisCity
														factory:[[game currentTransport] location]] completed:[[game currentTransport] type]])
								{
									
									[[game currentTransport] setDestination:[game currentCity]];
									[[game currentTransport] setDistanceRemaining:[game distanceBetweenFactory:[[game currentTransport] location]
																									   andCity:[game currentCity]]];
									
									[[game currentTransport] setRoute:[game distanceForFactory:[[game currentTransport] location]
																					   andCity:[game currentCity]]];
									
									[[game currentTransport] setLocation:nil];
									
									[self setDragged:nil];
									[self resetSelectionItems];
									[self setNeedsDisplay:YES];
									return;
								}
							}
						}
					}
					else
					{
						selectionPoint = centre;
						currentSelection = 0;
						
						// build a new icon wheel
						[self buildIconWheel:thisCity];
						[self setNeedsDisplay:YES];
						return;
					}
					break;
				}
			}
			
			// has an in transit transport been selected?
			// display transport
			for(loop = 0; loop < [thisPlayer transportCount]; loop++)
			{
				thisTransport = [thisPlayer getTransportAtIndex:loop];
				
				if([thisTransport route])
				{
					if([[thisTransport destination] isKindOfClass:[Factory class]])
					{
						if(NSPointInRect(centre,
										 NSMakeRect([[thisTransport route] getNodeAtIndex:[thisTransport distanceRemaining] forType:[thisTransport type]].x - 4 - 24 ,
													[[thisTransport route] getNodeAtIndex:[thisTransport distanceRemaining] forType:[thisTransport type]].y - 4 - 24,
													48, 48)))
						{
							[game setCurrentTransport:thisTransport];
							selectionPoint = centre;
							currentSelection = 0;
							
							// build a new icon wheel
							[self buildIconWheel:thisTransport];
							[self setNeedsDisplay:YES];
							return;
							
						}
					}
					else
					{
						
						if(NSPointInRect(centre,
										 NSMakeRect([[thisTransport route] getNodeAtIndex:[[thisTransport route] nodesCount:[thisTransport type]] - [thisTransport distanceRemaining] - 1 forType:[thisTransport type]].x - 4 - 24,
													[[thisTransport route] getNodeAtIndex:[[thisTransport route] nodesCount:[thisTransport type]] - [thisTransport distanceRemaining] - 1 forType:[thisTransport type]].y - 4 - 24,
													48, 48)))
						{
							[game setCurrentTransport:thisTransport];
							selectionPoint = centre;
							currentSelection = 0;
							
							// build a new icon wheel
							[self buildIconWheel:thisTransport];
							[self setNeedsDisplay:YES];
							return;
						}
					}
				}
			}
		}
	}
		
	[self resetSelectionItems];

	if([self dragged])
	{
		[self setDragged:nil];
	}
	
	[self setNeedsDisplay:YES];
}

- (void)displayTitle
{
	NSString *currVersionNumber = [[[NSBundle bundleForClass:[self class]]
        infoDictionary] objectForKey:@"CFBundleVersion"];
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
														  operation:NSCompositeSourceOver
														   fraction:0.8];	
	
	[[NSImage imageNamed:@"udg_mascot"] compositeToPoint:NSMakePoint(690, 80)
														  operation:NSCompositeSourceOver];
	
	[[NSImage imageNamed:@"logo"] compositeToPoint:NSMakePoint(20, 480)
											   operation:NSCompositeSourceOver];

	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	/*
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Industrial Revolution"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:18
			   colour:[NSColor yellowColor]];
	 */
	
	[self displayText:currVersionNumber
			  atPoint:NSMakePoint(600, 538)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"A uDevGames 2004 entry by Andrew Sage"
			  atPoint:NSMakePoint(60, 495)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	int yPos = 400;
	
	/*
	yPos -= [self displayText:@"This game is currently under development.\nDue to its development status there will be errors, incorrect graphics and missing game logic.\n\nUse the Check for Updates option from the application menu to check for the latest release."
			  atPoint:NSMakePoint(20, yPos)
				width:400
			   height:yPos - 100
			 fontSize:14
			   colour:[NSColor whiteColor]];
	 */
	
	[self displayText:@"Game Design and Graphics Copyright  2004 Andrew Sage"
			  atPoint:NSMakePoint(20, 99)
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"Music Copyright  2004 Gordon Murrison"
			  atPoint:NSMakePoint(20, 86)
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"Coding : Andrew Sage"
			  atPoint:NSMakePoint(20, 73)
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"Website : www.cloisterroom.com"
			  atPoint:NSMakePoint(20, 60)
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
}

- (void)displayCredits
{
	NSString *currVersionNumber = [[[NSBundle bundleForClass:[self class]]
        infoDictionary] objectForKey:@"CFBundleVersion"];
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
												 operation:NSCompositeSourceOver
												  fraction:0.5];	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Credits"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:18
			   colour:[NSColor yellowColor]];
	
	 [self displayText:@"Industrial Revolution"
			   atPoint:NSMakePoint(20, 560)
			  fontSize:20
				colour:[NSColor whiteColor]];
	 
	 [self displayText:currVersionNumber
			   atPoint:NSMakePoint(220, 570)
			  fontSize:10
				colour:[NSColor whiteColor]];
	 
	 [self displayText:@"A Work in Progress"
			   atPoint:NSMakePoint(20, 540)
			  fontSize:15
				colour:[NSColor whiteColor]];
	 
	 [self displayText:@"by Andrew Sage"
			   atPoint:NSMakePoint(20, 530)
			  fontSize:10
				colour:[NSColor whiteColor]];
	 
	 
	 
	 [self displayText:@"Game Design and Graphics Copyright  2004 Andrew Sage"
			   atPoint:NSMakePoint(20, 300)
			  fontSize:15
				colour:[NSColor whiteColor]];
	 
	 [self displayText:@"Coding : Andrew Sage"
			   atPoint:NSMakePoint(20, 280)
			  fontSize:15
				colour:[NSColor whiteColor]];
	 
	 [self displayText:@"Website : www.cloisterroom.com"
			   atPoint:NSMakePoint(20, 200)
			  fontSize:15
				colour:[NSColor whiteColor]];
	 
}

- (void)displayHelp
{
	Help* thisHelp;
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
														  operation:NSCompositeSourceOver
														   fraction:0.5];	
	
	[[NSImage imageNamed:@"logo"] compositeToPoint:NSMakePoint(-10, 510)
										 operation:NSCompositeSourceOver];
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Help"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:18
			   colour:[NSColor yellowColor]];
	
	thisHelp = [game getHelpAtIndex:[game currentHelp]];
	
	int yPos = 500;
	
	[self displayText:[thisHelp subject]
			  atPoint:NSMakePoint(40, yPos)
			 fontSize:20
			   colour:[NSColor whiteColor]];
	
	yPos -= 20;
	
	yPos -= 10;
	[self displayText:[thisHelp content]
			  atPoint:NSMakePoint(40, yPos)
				width:720
			   height:200
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
	
}

- (void)displayOffice
{
	int loop;
	int yPos = 500;
	int xPos = 10;
	Player* thisPlayer = [game currentPlayer];
	Factory* thisFactory = nil;
	Transport* thisTransport = nil;
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
														  operation:NSCompositeSourceOver
														   fraction:0.5];	
	
	[[NSImage imageNamed:@"logo"] compositeToPoint:NSMakePoint(-10, 510)
										 operation:NSCompositeSourceOver];
	
	
	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Office"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
		
	// display current cash flow
    [self displayText:[NSString stringWithFormat:@"Money: %06d",[thisPlayer money]]
			  atPoint:NSMakePoint(305, 100)
             fontSize:12
			   colour:[NSColor yellowColor]];
	
	
	yPos = 500;
	// display factories
	[self displayText:@"Factory"
			  atPoint:NSMakePoint(xPos, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Workforce"
			  atPoint:NSMakePoint(xPos + 60, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Stock"
			  atPoint:NSMakePoint(xPos + 120, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Output/Person"
			  atPoint:NSMakePoint(xPos + 180, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Size"
			  atPoint:NSMakePoint(xPos + 260, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Running Cost"
			  atPoint:NSMakePoint(xPos + 320, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	yPos -= 10;
	
	
	// display factories
	for(loop = 0; loop < [game factoryCount]; loop++)
	{
		thisFactory = [game getFactoryAtIndex:loop];
		
		if([thisFactory owner] == thisPlayer)
		{
			[self displayText:[thisFactory description]
					  atPoint:NSMakePoint(xPos, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			
			if([thisFactory turnsToBuild] > 0)
			{
				[self displayText:[NSString stringWithFormat:@"Under Construction : %d turns until completion", [thisFactory turnsToBuild]]
						  atPoint:NSMakePoint(xPos + 60, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
			}
			else
			{
				// only display details if the factory is built
				[self displayText:[NSString stringWithFormat:@"%d / %d",
					[thisFactory workforce],
					[thisFactory workforceMax]]
						  atPoint:NSMakePoint(xPos + 60, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
				
				[self displayText:[NSString stringWithFormat:@"%d / %d",
					[thisFactory stock],
					[thisFactory stockMax]]
						  atPoint:NSMakePoint(xPos + 120, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
				
				[self displayText:[NSString stringWithFormat:@"%d", [thisFactory outputRatio]]
						  atPoint:NSMakePoint(xPos + 180, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
				
				[self displayText:[NSString stringWithFormat:@"%d", [thisFactory size]]
						  atPoint:NSMakePoint(xPos + 260, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
				
				[self displayText:[NSString stringWithFormat:@"%d", [thisFactory workforce] * [thisFactory runningCost]]
						  atPoint:NSMakePoint(xPos + 320, yPos)
						 fontSize:10
						   colour:[NSColor whiteColor]];
			}
			
			yPos -= 10;
		}
		
	}	
	
	yPos -= 50;
	// display transport
	[self displayText:@"Transport"
			  atPoint:NSMakePoint(xPos, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Load"
			  atPoint:NSMakePoint(xPos + 100, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Miles"
			  atPoint:NSMakePoint(xPos + 150, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Destination"
			  atPoint:NSMakePoint(xPos + 200, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	[self displayText:@"Location"
			  atPoint:NSMakePoint(xPos + 280, yPos)
			 fontSize:10
			   colour:[NSColor yellowColor]];
	
	yPos -= 12;
	
		
	for(loop = 0; loop < [thisPlayer transportCount]; loop++)
	{
		thisTransport = [thisPlayer getTransportAtIndex:loop];
		
		
		if([thisTransport turnsToBuild] > 0)
		{
			[self displayText:[NSString stringWithFormat:@"%@ : Under Construction : %d turns until completion",
				[thisTransport typeAsText],
				[thisTransport turnsToBuild]]
					  atPoint:NSMakePoint(xPos, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
		}
		else
		{
			[self displayText:[NSString stringWithFormat:@"%@",
				[thisTransport typeAsText]]
					  atPoint:NSMakePoint(xPos, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			[self displayText:[NSString stringWithFormat:@"%d / %d",
				[thisTransport load],
				[thisTransport capacity]]
					  atPoint:NSMakePoint(xPos + 100, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			[self displayText:[NSString stringWithFormat:@"%d",
				[thisTransport distanceRemaining]]
					  atPoint:NSMakePoint(xPos + 150, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			[self displayText:[NSString stringWithFormat:@"%@",
				[thisTransport destinationName]]
					  atPoint:NSMakePoint(xPos + 200, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			[self displayText:[NSString stringWithFormat:@"%@",
				[thisTransport locationName]]
					  atPoint:NSMakePoint(xPos + 280, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
		}

		yPos -= 10;
	}
	
	// display the current task
	[self drawCurrentTask];
}

- (void)displayStats
{
	int loop;
	Player* thisPlayer = [game currentPlayer];
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
														  operation:NSCompositeSourceOver
														   fraction:0.5];	
	
	[[NSImage imageNamed:@"logo"] compositeToPoint:NSMakePoint(-10, 510)
										 operation:NSCompositeSourceOver];
	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Statistics"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
	[self displayText:[NSString stringWithFormat:@"Turn: %d", [thisPlayer statsCount] + 1]
			  atPoint:NSMakePoint(5, 60)
			 fontSize:12
			   colour:[NSColor yellowColor]];
	

	NSBezierPath *pathMoney = [NSBezierPath bezierPath]; // path for drawing the money graph
	NSBezierPath *pathMoneyOut = [NSBezierPath bezierPath]; // path for drawing the money out graph
	NSBezierPath *pathMoneyIn = [NSBezierPath bezierPath]; // path for drawing the money in graph

	
	Stats* thisStats;
	
	[pathMoney moveToPoint:NSMakePoint(100, 100)];
	[pathMoneyOut moveToPoint:NSMakePoint(100, 300)];
	[pathMoneyIn moveToPoint:NSMakePoint(100, 300)];

	// get max and min values for scaling
	int moneyInOutMax = 10;
	int moneyInOutMin = 999999;
	int moneyMax = 10000; //[thisPlayer money];
	
	// get the horizontal step
	int step = 600 / [thisPlayer statsCount] + 1;
	
	
	for(loop = 0; loop < [thisPlayer statsCount]; loop++)
	{
		thisStats = [thisPlayer getStatsAtIndex:loop];
		
		if([thisStats moneyIn] > moneyInOutMax)
		{
			moneyInOutMax = [thisStats moneyIn];
		}
		
		if([thisStats moneyOut] > moneyInOutMax)
		{
			moneyInOutMax = [thisStats moneyOut];
		}
		
		if([thisStats moneyIn] < moneyInOutMin)
		{
			moneyInOutMin = [thisStats moneyIn];
		}
		
		if([thisStats moneyOut] < moneyInOutMin)
		{
			moneyInOutMin = [thisStats moneyOut];
		}
		
		if([thisStats moneyTotal] > moneyMax)
		{
			moneyMax = [thisStats moneyTotal];
		}
	}
	
	float scale = moneyInOutMax / 200.00;
	float scaleMoney = moneyMax / 100.00;
	
	[self displayText:@"000000"
			  atPoint:NSMakePoint(55, 300)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"Money"
			  atPoint:NSMakePoint(55, 515)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"000000"
			  atPoint:NSMakePoint(55, 95)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	[self displayText:@"Money"
			  atPoint:NSMakePoint(55, 215)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	NSBezierPath *pathLine = [NSBezierPath bezierPath]; // path for drawing the  graph lines
	
	// money in out
	[pathLine moveToPoint:NSMakePoint(100, 300)];
	[pathLine lineToPoint:NSMakePoint(700, 300)];
	
	// money
	[pathLine moveToPoint:NSMakePoint(100, 100)];
	[pathLine lineToPoint:NSMakePoint(700, 100)];

	for(loop = 0; loop < 10; loop++)
	{
		// money in out lines
		[self displayText:[NSString stringWithFormat:@"%06d", (moneyInOutMax / 10) * (loop + 1)]
				  atPoint:NSMakePoint(55, 300 + (20 * (loop + 1)) - 5)
				 fontSize:10
				   colour:[NSColor whiteColor]];
		
		[pathLine moveToPoint:NSMakePoint(100, 300 + (20 * (loop + 1)))];
		[pathLine lineToPoint:NSMakePoint(700, 300 + (20 * (loop + 1)))];
		
		// money lines
		[self displayText:[NSString stringWithFormat:@"%06d", (moneyMax / 10) * (loop + 1)]
				  atPoint:NSMakePoint(55, 100 + (10 * (loop + 1)) - 5)
				 fontSize:10
				   colour:[NSColor whiteColor]];
		
		[pathLine moveToPoint:NSMakePoint(100, 100 + (10 * (loop + 1)))];
		[pathLine lineToPoint:NSMakePoint(700, 100 + (10 * (loop + 1)))];
	}
	
	[[NSColor colorWithDeviceRed:1
						   green:1
							blue:1
						   alpha:0.2] set];
	[pathLine setLineWidth:1];
	[pathLine stroke];
	
	for(loop = 0; loop < [thisPlayer statsCount]; loop++)
	{
		thisStats = [thisPlayer getStatsAtIndex:loop];
		
		[pathMoney lineToPoint:NSMakePoint(100 + (loop * step), ([thisStats moneyTotal] / scaleMoney) + 100)];
		[pathMoneyOut lineToPoint:NSMakePoint(100 + (loop * step), ([thisStats moneyOut] / scale) + 300)];
		[pathMoneyIn lineToPoint:NSMakePoint(100 + (loop * step), ([thisStats moneyIn] / scale) + 300)];
	}
	
	// add current money
	[pathMoney lineToPoint:NSMakePoint(100 + (loop * step), ([thisPlayer money] / scaleMoney) + 100)];
	
		
	// create key
	[pathMoneyIn moveToPoint:NSMakePoint(110, 280)];
	[pathMoneyIn lineToPoint:NSMakePoint(150, 280)];
	
	[self displayText:@"Money In"
			  atPoint:NSMakePoint(55, 275)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	
	[pathMoneyOut moveToPoint:NSMakePoint(110, 260)];
	[pathMoneyOut lineToPoint:NSMakePoint(150, 260)];
	
	[self displayText:@"Money Out"
			  atPoint:NSMakePoint(55, 255)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	[pathMoney moveToPoint:NSMakePoint(110, 240)];
	[pathMoney lineToPoint:NSMakePoint(150, 240)];
	
	[self displayText:@"Money"
			  atPoint:NSMakePoint(55, 235)
			 fontSize:10
			   colour:[NSColor whiteColor]];
		
	[[NSColor greenColor] set];
	[pathMoneyIn setLineWidth:2];
	[pathMoneyIn stroke];
	
	[[NSColor redColor] set];
	[pathMoneyOut setLineWidth:2];
	[pathMoneyOut stroke];
	
	[[NSColor yellowColor] set];
	[pathMoney stroke];
}

- (void)displayMap
{
	City* thisCity = nil;
	Factory* thisFactory = nil;
	Distance* thisDistance = nil;
	Player* thisPlayer = [game currentPlayer];
	Transport* thisTransport = nil;

	int yPos;
	int loop;
	
	[[NSColor colorWithDeviceRed:0
						   green:0.5
							blue:0
						   alpha:1] set];
		
	NSRectFill(windowRect);
	
	
	int x, y;
	
	for(x = 0; x < 800; x += 256)
	{
		for(y = 0; y < 600; y += 256)
		{
			[imageGrassTexture compositeToPoint:NSMakePoint(x, y) operation:NSCompositeSourceOver fraction:0.5];
		}
	}
	
	NSBezierPath *pathRoad = [NSBezierPath bezierPath]; // path for drawing the connection routes
	NSBezierPath *pathFactories = [NSBezierPath bezierPath]; // path for the drawing the factories
	NSBezierPath *pathSelected = [NSBezierPath bezierPath]; // path of selected objects
	NSBezierPath *pathNodes = [NSBezierPath bezierPath]; // path of route nodes
	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Map"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
	// display current cash flow
    [self displayText:[NSString stringWithFormat:@"Money: %06d",[thisPlayer money]]
			  atPoint:NSMakePoint(305, 100)
             fontSize:12
			   colour:[NSColor yellowColor]];
	
	

	int factoryInfoX = 0;
	int factoryInfoY = 255;


	// only show factory details if a factory is selected
	{
		// draw factory details background box
		[[[NSColor colorWithDeviceRed:0.0 green:0.1 blue:0.1 alpha:1.0] colorWithAlphaComponent:0.2] set];
		
		NSRectFillUsingOperation(NSMakeRect(factoryInfoX - 2, factoryInfoY - 200 - 2, 150 + 4, 200 + 4), NSCompositeSourceOver);
		NSRectFillUsingOperation(NSMakeRect(factoryInfoX, factoryInfoY - 200, 150, 200), NSCompositeSourceOver);
	
		yPos = factoryInfoY - 16;
		
		[self displayText:@"Current Factory"
				  atPoint:NSMakePoint(factoryInfoX + 4, yPos)
				 fontSize:10
				   colour:[NSColor whiteColor]];
		
		yPos -= 12;
		
		if([game currentFactory])
		{
			//[[NSImage imageNamed:@"inside_factory"] compositeToPoint:NSMakePoint(20, 460) operation:NSCompositeSourceOver fraction:0.5];
			
			[self displayText:[NSString stringWithFormat:@"Factory: %@", [[game currentFactory] description]]
					  atPoint:NSMakePoint(factoryInfoX + 4, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			yPos -= 12;
			
			[self displayText:[NSString stringWithFormat:@"Owned by %@", [[[game currentFactory] owner] name]]
					  atPoint:NSMakePoint(factoryInfoX + 4, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			yPos -= 12;
			
			if([[game currentFactory] turnsToBuild] > 0)
			{
				[self displayText:[NSString stringWithFormat:@"Under Construction\n%d turns until completion", [[game currentFactory] turnsToBuild]]
						  atPoint:NSMakePoint(factoryInfoX + 4, yPos - 20)
						 fontSize:10
						   colour:[NSColor whiteColor]];
			}
			else
			{
				
				[[NSImage imageNamed:@"factory_details"] compositeToPoint:NSMakePoint(factoryInfoX, factoryInfoY - 240) operation:NSCompositeSourceOver];
				
				yPos = factoryInfoY - 60;
				
				[self displayText:[NSString stringWithFormat:@"%d", [[game currentFactory] size]]
						  atPoint:NSMakePoint(factoryInfoX + 40, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 30;
				
				// only display details if the factory is built
				[self displayText:[NSString stringWithFormat:@"%d / %d (max)",
					[[game currentFactory] workforce],
					[[game currentFactory] workforceMax]]
						  atPoint:NSMakePoint(factoryInfoX + 40, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 30;
				
				[self displayText:[NSString stringWithFormat:@"%d / %d (max)",
					[[game currentFactory] stock],
					[[game currentFactory] stockMax]]
						  atPoint:NSMakePoint(factoryInfoX + 40, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 30;
				
				
				[self displayText:[NSString stringWithFormat:@"%d", [[game currentFactory] outputRatio]]
						  atPoint:NSMakePoint(factoryInfoX + 40, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 30;
				
				[self displayText:[NSString stringWithFormat:@"%d", [[game currentFactory] workforce] * [[game currentFactory] runningCost]]
						  atPoint:NSMakePoint(factoryInfoX + 40, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;
				
			}
		}
		
	}
	
	// draw connecting lines
	for(loop = 0; loop < [game distanceCount]; loop++)
	{
		thisDistance = [game getDistanceAtIndex:loop];
		
				
		// check to see if factory is owned by player
		
		//if([[thisDistance factory] owner] == thisPlayer)
		{
			int nodeLoop;
			
			[pathRoad removeAllPoints];
			[pathNodes removeAllPoints];

			[pathRoad moveToPoint:[thisDistance getNodeAtIndex:0
													   forType:NETWORK_ROAD]];
						
			for(nodeLoop = 1; nodeLoop < [thisDistance built:NETWORK_ROAD] + 1; nodeLoop++)
			{
				NSPoint thisPoint = [thisDistance getNodeAtIndex:nodeLoop
														 forType:NETWORK_ROAD];
				
				NSPoint prevPoint = [thisDistance getNodeAtIndex:nodeLoop - 1
														 forType:NETWORK_ROAD];
				
				[pathRoad lineToPoint:thisPoint];
				
				
				/*
				if(nodeLoop % 2 == 1)
				{
					[pathRoad curveToPoint:thisPoint
							 controlPoint1:NSMakePoint(prevPoint.x + 5, prevPoint.y - 20)
							 controlPoint2:NSMakePoint(thisPoint.x + 5, thisPoint.y + 20)];
				}
				else
				{
					[pathRoad curveToPoint:thisPoint
							 controlPoint1:NSMakePoint(prevPoint.x - 5, prevPoint.y - 20)
							 controlPoint2:NSMakePoint(thisPoint.x - 5, thisPoint.y + 20)];
					
				}
				*/
				
				[pathNodes appendBezierPathWithOvalInRect:NSMakeRect(thisPoint.x - 4,
																		 thisPoint.y - 4,
																		 8, 8)];
			}

			[[NSColor orangeColor] set];
		
			[pathRoad stroke];
			[pathNodes fill];
			
			
			[pathRoad removeAllPoints];
			[pathNodes removeAllPoints];
			
			[pathRoad moveToPoint:[thisDistance getNodeAtIndex:0
													   forType:NETWORK_RAIL]];
			
			for(nodeLoop = 1; nodeLoop < [thisDistance built:NETWORK_RAIL] + 1; nodeLoop++)
			{
				NSPoint thisPoint = [thisDistance getNodeAtIndex:nodeLoop
														 forType:NETWORK_RAIL];
				[pathRoad lineToPoint:thisPoint];
				
				[pathNodes appendBezierPathWithOvalInRect:NSMakeRect(thisPoint.x - 4,
																		 thisPoint.y - 4,
																		 8, 8)];
			}
			
			[[NSColor blackColor] set];
			
			[pathRoad stroke];
			[pathNodes fill];


			[pathRoad removeAllPoints];
			[pathNodes removeAllPoints];
			
			[pathRoad moveToPoint:[thisDistance getNodeAtIndex:0
													   forType:NETWORK_CANAL]];
			
			for(nodeLoop = 1; nodeLoop < [thisDistance built:NETWORK_CANAL] + 1; nodeLoop++)
			{
				NSPoint thisPoint = [thisDistance getNodeAtIndex:nodeLoop
														 forType:NETWORK_CANAL];
				
				NSPoint prevPoint = [thisDistance getNodeAtIndex:nodeLoop - 1
														 forType:NETWORK_CANAL];
				
				if(nodeLoop % 2 == 0)
				{
					[pathRoad curveToPoint:thisPoint
							 controlPoint1:NSMakePoint(prevPoint.x + 5, prevPoint.y - 10)
							 controlPoint2:NSMakePoint(thisPoint.x + 5, thisPoint.y + 10)];
				}
				else
				{
					[pathRoad curveToPoint:thisPoint
							 controlPoint1:NSMakePoint(prevPoint.x - 5, prevPoint.y - 10)
							 controlPoint2:NSMakePoint(thisPoint.x - 5, thisPoint.y + 10)];
					
				}
				
				[pathNodes appendBezierPathWithOvalInRect:NSMakeRect(thisPoint.x - 4,
																	 thisPoint.y - 4,
																	 8, 8)];
				
			}
			
			[[NSColor blueColor] set];
			
			[pathRoad stroke];
			[pathNodes fill];
		
			thisFactory = [thisDistance factory];
			
			if([self dragged] && NSPointInRect(mousePoint, NSMakeRect([thisFactory positionMap].x - 24,
													[thisFactory positionMap].y - 24,
													48, 48)))
			{
				NSBezierPath *pathHighlight = [NSBezierPath bezierPath]; // path for drawing the highlight
				
				[pathHighlight appendBezierPathWithOvalInRect:NSMakeRect([thisFactory positionMap].x - 24,
																		 [thisFactory positionMap].y - 24,
																		 48, 48)];
				
				[pathHighlight fill];
			}
			
			[[thisFactory image] compositeToPoint:NSMakePoint([thisFactory positionMap].x - 24,
															  [thisFactory positionMap].y - 24) operation:NSCompositeSourceOver];
			
			
		
			[self displayText:[thisFactory name]
					  atPoint:[thisFactory positionMap]
					 fontSize:10
					   colour:[NSColor yellowColor]];
			
			if(thisFactory == [game currentFactory])
			{
				[pathSelected appendBezierPathWithOvalInRect:NSMakeRect([thisFactory positionMap].x - 5,
																		 [thisFactory positionMap].y - 5,
																		 10, 10)];
			}
			else
			{
				[pathFactories appendBezierPathWithOvalInRect:NSMakeRect([thisFactory positionMap].x - 5,
																		 [thisFactory positionMap].y - 5,
																		 10, 10)];
				
			}
		}
	}
	
	
	[pathFactories fill];
	[pathFactories removeAllPoints];
	
	
	
	// display the cities
	for(loop = 0; loop < [game cityCount]; loop++)
	{
		thisCity = [game getCityAtIndex:loop];
		
		if([self dragged] && NSPointInRect(mousePoint, NSMakeRect([thisCity positionMap].x - 24,
												[thisCity positionMap].y - 24,
												48, 48)))
		{
			if([[self dragged] isKindOfClass:[Action class]])
			{
				//Action* thisAction = [self dragged];
				
				if([game connectionBetweenCity:thisCity
									 factory:[game currentFactory]])
				{
					NSBezierPath *pathHighlight = [NSBezierPath bezierPath]; // path for drawing the highlight
					
					[pathHighlight appendBezierPathWithOvalInRect:NSMakeRect([thisCity positionMap].x - 24,
																			 [thisCity positionMap].y - 24,
																			 48, 48)];
					
					[pathHighlight fill];
				}
			}
			
			if([[self dragged] isKindOfClass:[Transport class]])
			{
				Transport* thisTransport = [self dragged];
				if([[game connectionBetweenCity:thisCity
										factory:[thisTransport location]] completed:[thisTransport type]])
				{
					NSBezierPath *pathHighlight = [NSBezierPath bezierPath]; // path for drawing the highlight
					
					[pathHighlight appendBezierPathWithOvalInRect:NSMakeRect([thisCity positionMap].x - 24,
																			 [thisCity positionMap].y - 24,
																			 48, 48)];
					
					[pathHighlight fill];
				}
			}
		}
		
		[[thisCity image] compositeToPoint:NSMakePoint([thisCity positionMap].x - 24,
														  [thisCity positionMap].y - 24) operation:NSCompositeSourceOver];
		
		
		[self displayText:[NSString stringWithFormat:@"%@ (%d)",
			[thisCity name],
			[thisCity price]]
				  atPoint:[thisCity positionMap]
				 fontSize:10
				   colour:[NSColor yellowColor]];
		
		if(thisCity == [game currentCity])
		{
			[pathSelected appendBezierPathWithOvalInRect:NSMakeRect([thisCity positionMap].x - 5,
																	 [thisCity positionMap].y - 5,
																	 10, 10)];
		}
		else
		{
			[pathFactories appendBezierPathWithOvalInRect:NSMakeRect([thisCity positionMap].x - 5,
																	 [thisCity positionMap].y - 5,
																	 10, 10)];
		}
	}
	
	[[NSColor redColor] set];
	
	[pathSelected fill];
	[pathSelected removeAllPoints];
	
	
	[[NSColor brownColor] set];
	

	
	[pathFactories fill];
	
	[pathRoad removeAllPoints];
	
	// display transport
	for(loop = 0; loop < [thisPlayer transportCount]; loop++)
	{
		thisTransport = [thisPlayer getTransportAtIndex:loop];
		
		if([thisTransport route] && [thisTransport distanceRemaining] != [[thisTransport route] distance:[thisTransport type]])
		{
			if([[thisTransport destination] isKindOfClass:[Factory class]])
			{
				[[thisTransport image] compositeToPoint:NSMakePoint([[thisTransport route] getNodeAtIndex:[thisTransport distanceRemaining] forType:[thisTransport type]].x - 4 - 24 ,
																	[[thisTransport route] getNodeAtIndex:[thisTransport distanceRemaining] forType:[thisTransport type]].y - 4 - 24)
																	operation:NSCompositeSourceOver
											   fraction:1.0];
			}
			else
			{

				[[thisTransport image] compositeToPoint:NSMakePoint([[thisTransport route] getNodeAtIndex:[[thisTransport route] nodesCount:[thisTransport type]] - [thisTransport distanceRemaining] - 1 forType:[thisTransport type]].x - 4 - 24,
																	[[thisTransport route] getNodeAtIndex:[[thisTransport route] nodesCount:[thisTransport type]] - [thisTransport distanceRemaining] - 1 forType:[thisTransport type]].y - 4 - 24)
											  operation:NSCompositeSourceOver
											   fraction:1.0];
			}
		}
		
		if([thisTransport location])
		{
			[[thisTransport image] compositeToPoint:NSMakePoint([[thisTransport location] positionMap].x - 4 - 24,
																[[thisTransport location] positionMap].y - 4 - 24)
										  operation:NSCompositeSourceOver
										   fraction:1.0];
			
			
			[thisTransport setPositionMap:NSMakePoint([[thisTransport location] positionMap].x - 4 - 24,
													  [[thisTransport location] positionMap].y - 4 - 24)];
			
			
			if([[thisTransport location] isKindOfClass:[Factory class]])
			{
				[pathRoad appendBezierPathWithOvalInRect:NSMakeRect([[thisTransport location] positionMap].x - 4,
															   [[thisTransport location] positionMap].y - 4,
															   8, 8)];
				
			}
			else
			{
				
				[pathRoad appendBezierPathWithOvalInRect:NSMakeRect([[thisTransport location] positionMap].x - 4,
															  [[thisTransport location] positionMap].y - 4,
															  8, 8)];
				
			}
		}
		else
		{
		}
	}
		
	[[NSColor darkGrayColor] set];
	[pathRoad fill];
	
	int workersInfoX = 0;
	int workersInfoY = 484;
	
	/*
	// display details about the workers
	{
		// draw workers details background box
		[[[NSColor colorWithDeviceRed:0.0 green:0.1 blue:0.1 alpha:1.0] colorWithAlphaComponent:0.2] set];
		
		NSRectFillUsingOperation(NSMakeRect(workersInfoX - 2, workersInfoY - 134 + 12 - 2, 150 + 4, 134 - 12 + 4), NSCompositeSourceOver);
		NSRectFillUsingOperation(NSMakeRect(workersInfoX, workersInfoY - 134 + 12 , 150, 134 - 12), NSCompositeSourceOver);
		
		yPos = workersInfoY - 16;
		[self displayText:@"Workers"
				  atPoint:NSMakePoint(workersInfoX + 4, yPos)
				 fontSize:10
				   colour:[NSColor whiteColor]];
	}
	 */
	
	int transportInfoX = 0;
	int transportInfoY = 600;
	
	// display details about the current transport
	{
		// draw transport details background box
		[[[NSColor colorWithDeviceRed:0.0 green:0.1 blue:0.1 alpha:1.0] colorWithAlphaComponent:0.2] set];
		
		NSRectFillUsingOperation(NSMakeRect(transportInfoX - 2, transportInfoY - 112 - 2, 200 + 4, 112 + 4), NSCompositeSourceOver);
		NSRectFillUsingOperation(NSMakeRect(transportInfoX, transportInfoY - 112, 200, 112), NSCompositeSourceOver);
		
		yPos = transportInfoY - 16;
		[self displayText:@"Current Transport"
				  atPoint:NSMakePoint(transportInfoX + 4, yPos)
				 fontSize:10
				   colour:[NSColor whiteColor]];
		
		if([game currentTransport] != nil)
		{
			[[[game currentTransport] image] compositeToPoint:NSMakePoint(transportInfoX + 130, transportInfoY - 50)
													operation:NSCompositeSourceOver
													 fraction:1.0];
			
			yPos -= 12;
			
			[self displayText:[NSString stringWithFormat:@"%@",
				[[game currentTransport] typeAsText]]
					  atPoint:NSMakePoint(transportInfoX + 4, yPos)
					 fontSize:12
					   colour:[NSColor whiteColor]];

			yPos -= 12;

			[self displayText:[NSString stringWithFormat:@"Load %d / %d Max",
				[[game currentTransport] load],
				[[game currentTransport] capacity]]
					  atPoint:NSMakePoint(transportInfoX + 4, yPos)
					 fontSize:12
					   colour:[NSColor whiteColor]];

			yPos -= 16;

			if([[game currentTransport] destination])
			{
				[self displayText:[NSString stringWithFormat:@"From: %@",
					[[game currentTransport] fromName]]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;
				
				[self displayText:[NSString stringWithFormat:@"Destination: %@",
					[[game currentTransport] destinationName]]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;
				
				[self displayText:[NSString stringWithFormat:@"Distance remaining: %d",
					[[game currentTransport] distanceRemaining]]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];

				yPos -= 12;
				
				[self displayText:[NSString stringWithFormat:@"ETA: %d turn(s)",
					[[game currentTransport] distanceRemaining] / [[game currentTransport] speed]]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;
				
				[self displayText:[NSString stringWithFormat:@"Auto Return: %@",
			  [[game currentTransport] autoReturn] ? @"Yes" : @"No"]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;

				
			}

			if([[game currentTransport] location])
			{
				[self displayText:[NSString stringWithFormat:@"Location: %@",
					[[game currentTransport] locationName]]
						  atPoint:NSMakePoint(transportInfoX + 4, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
			}
		}
		
	}
	
	int cityInfoX = 0;
	int cityInfoY = 359;
	
	// display details about the current city
	{
		// draw city details background box
		[[[NSColor colorWithDeviceRed:0.0 green:0.1 blue:0.1 alpha:1.0] colorWithAlphaComponent:0.2] set];
		
		NSRectFillUsingOperation(NSMakeRect(cityInfoX - 2, cityInfoY - 100 - 2, 150 + 4, 100 + 4), NSCompositeSourceOver);
		NSRectFillUsingOperation(NSMakeRect(cityInfoX, cityInfoY - 100, 150, 100), NSCompositeSourceOver);
		
		yPos = cityInfoY - 16;
		
		[self displayText:@"Current City"
				  atPoint:NSMakePoint(cityInfoX + 4, yPos)
				 fontSize:10
				   colour:[NSColor whiteColor]];
		
		if([game currentCity] != nil)
		{
			yPos -= 12;
			
			[self displayText:[[game currentCity] name]
					  atPoint:NSMakePoint(cityInfoX + 4, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
			yPos -= 12;
			
			[self displayText:[NSString stringWithFormat:@"Price paid per unit: %d", [[game currentCity] price]]
					  atPoint:NSMakePoint(cityInfoX + 4, yPos)
					 fontSize:10
					   colour:[NSColor whiteColor]];
			
		}
	}

	
	// display the current task
	[self drawCurrentTask];

	/*
	[self displayText:@"Map"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor whiteColor]];
	 */
}

- (void)displayShop
{
	Player* thisPlayer = [game currentPlayer];

	int yPos;
	ShopItem* item;

	
	yPos = 100;
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[[NSImage imageNamed:@"inventions_background"] compositeToPoint:NSMakePoint(0,0)
														  operation:NSCompositeSourceOver
														   fraction:0.5];	
	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Shop"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
    
    // draw inventory background box
    [[[NSColor colorWithDeviceRed:0.0 green:0.0 blue:0.5 alpha:1.0] colorWithAlphaComponent:0.5] set];
	
    // shop item background
    NSRectFillUsingOperation(NSMakeRect(10, 190, 280, 160), NSCompositeSourceOver);
    NSRectFillUsingOperation(NSMakeRect(10, 355, 280, 245), NSCompositeSourceOver);
	
    // display current cash flow
    [self displayText:[NSString stringWithFormat:@"Money: %06d",[thisPlayer money]]
                  atPoint:NSMakePoint(305, 100)
             fontSize:12
			   colour:[NSColor yellowColor]];

	
    // display current item
    item = [game getShopItemAt:[game currentShopItem]];
	
    // work out if the item is affordable
    double fade;
    if([thisPlayer canAfford:[item cost]])
    {
        fade = 0.25;
    }
    else
    {
        fade = 1.0;
    }
	
    //[[item getImage] compositeToPoint:NSMakePoint(520, 260) operation:NSCompositeSourceOver fraction:fade];
	
	
    [self displayText:[item title]
                   atPoint:NSMakePoint(15, 320)
             fontSize:20
			   colour:[NSColor yellowColor]];

	[self displayText:[item details]
			  atPoint:NSMakePoint(15, 320)
				width:270
			   height:100
			 fontSize:12
			   colour:[NSColor yellowColor]];
    
    [self displayText:@"Cost"
                  atPoint:NSMakePoint(15, 210)
             fontSize:12
			   colour:[NSColor yellowColor]];

    [self displayText:[NSString stringWithFormat:@"%d",[item cost]]
                   atPoint:NSMakePoint(15, 190)
             fontSize:20
			   colour:[NSColor yellowColor]];
	
	
	[self displayText:@"Delivery Time"
			  atPoint:NSMakePoint(165, 210)
             fontSize:12
			   colour:[NSColor yellowColor]];
	
    [self displayText:[NSString stringWithFormat:@"%d",[item turns]]
			  atPoint:NSMakePoint(165, 190)
             fontSize:20
			   colour:[NSColor yellowColor]];

	// display the current task
	[self drawCurrentTask];
	
}

- (void)displayEndTurn
{
	int yPos;
	int xPos;
	int loop;
	
	Player* thisPlayer = [game currentPlayer];

	[[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0] set];
	NSRectFill(windowRect);
	
	NSImage* imageNewspaperTexture = [NSImage imageNamed:@"newspaper_texture"];

	int x, y;
	
	for(x = 0; x < 800; x += 64)
	{
		for(y = 0; y < 600; y += 64)
		{
			[imageNewspaperTexture compositeToPoint:NSMakePoint(x, y)
										  operation:NSCompositeSourceOver
										   fraction:0.5];
		}
	}
	
	// heading
	[[NSImage imageNamed:@"newspaper_heading"] compositeToPoint:NSMakePoint(0, 400) operation:NSCompositeSourceOver];
		
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"End of Turn"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
	[self displayText:[NSString stringWithFormat:@"Issue: %5d", [thisPlayer statsCount]]
			  atPoint:NSMakePoint(5, 550)
			 fontSize:15
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	[self displayText:@"Financial News"
			  atPoint:NSMakePoint(5, 500)
			 fontSize:15
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	[self displayText:[NSString stringWithFormat:@"Money In   : %05d", [[thisPlayer getLastStats] moneyIn]]
			  atPoint:NSMakePoint(5, 485)
			 fontSize:12
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	[self displayText:[NSString stringWithFormat:@"Money Out: %05d", [[thisPlayer getLastStats] moneyOut]]
			  atPoint:NSMakePoint(5, 473)
			 fontSize:12
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	[self displayText:[NSString stringWithFormat:@"%@     : %05d",
		 [[thisPlayer getLastStats] moneyIn] - [[thisPlayer getLastStats] moneyOut] > 0 ? @"Profit   " : @"Loss    ",
		abs([[thisPlayer getLastStats] moneyIn] - [[thisPlayer getLastStats] moneyOut])]
			  atPoint:NSMakePoint(5, 461)
			 fontSize:12
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	
	[self displayText:@"Stock News"
			  atPoint:NSMakePoint(5, 430)
			 fontSize:15
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	[self displayText:[NSString stringWithFormat:@"%d unit%s produced",
		[[thisPlayer getLastStats] produced],
		[[thisPlayer getLastStats] produced] == 1 ? "" : "s"
		]
			  atPoint:NSMakePoint(5, 415)
			 fontSize:12
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	
	
	// display any message
	yPos = 490;
	
	[self displayText:@"News this Turn"
			  atPoint:NSMakePoint(200, yPos)
			 fontSize:18
			   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
	yPos -= 20;
	
	int col1Ypos = 470;
	int col2Ypos = 470;
	
	for(loop = 0; loop < [game messageCount]; loop++)
	{
		if(loop % 2 == 0)
		{
			xPos = 200;
			col1Ypos -= [self displayText:[game getMessageAtIndex:loop]
					  atPoint:NSMakePoint(xPos, col1Ypos)
						width:250
					   height:col1Ypos - 100
					 fontSize:12
					   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
			
			col1Ypos -= 10;
		}
		else
		{
			xPos = 500;
			col2Ypos -= [self displayText:[game getMessageAtIndex:loop]
					  atPoint:NSMakePoint(xPos, col2Ypos)
						width:250
					   height:col2Ypos - 100
					 fontSize:12
					   colour:[NSColor colorWithDeviceRed:0.5 green:0.3 blue:0.2 alpha:1.0]];
			
			col2Ypos -= 10;
		}
		
		
		
		if(loop % 2 == 1)
		{
			yPos -= 14;
		}
	}
}

- (void)displayInventions
{
	int loop, subLoop;
	int yPos = 550;
	int xPos = 40;
	
	Player* thisPlayer = [game currentPlayer];
	Invention* thisInvention = [thisPlayer getInventionAtIndex:[game currentInvention]];
	
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);

	[[NSImage imageNamed:@"title_screen"] compositeToPoint:NSMakePoint(0,15)
														  operation:NSCompositeSourceOver
														   fraction:0.5];	
	
	[[NSImage imageNamed:@"logo"] compositeToPoint:NSMakePoint(-10, 510)
										 operation:NSCompositeSourceOver];
	
	// button backing
	[imageWoodPanel compositeToPoint:NSMakePoint(0, 0) operation:NSCompositeSourceOver];
	[imageAboveWoodPanel compositeToPoint:NSMakePoint(300, 55) operation:NSCompositeSourceOver];
	[self displayText:@"Inventions"
			  atPoint:NSMakePoint(310, 60)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
	// display current research budget
    [self displayText:[NSString stringWithFormat:@"Research Resources: %06d",[thisPlayer researchBudget]]
			  atPoint:NSMakePoint(305, 100)
             fontSize:12
			   colour:[NSColor yellowColor]];
	
	yPos = 450;
	[self displayText:@"Technology Points"
			  atPoint:NSMakePoint(xPos, yPos + 20)
			 fontSize:20
			   colour:[NSColor yellowColor]];

	
	// display tech levels
	for(loop = 0; loop < TECH_COUNT; loop++)
	{
		[self displayText:[game technologyName:loop]
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		
		[self displayText:[NSString stringWithFormat:@"%d / %d", [thisPlayer tech:loop],
			[thisPlayer research:loop]]
				  atPoint:NSMakePoint(xPos + 80, yPos)
				 fontSize:12
				   colour:[NSColor whiteColor]];
		
		yPos -= 12;
	}
	
	yPos -= 20;
	
	[self displayText:@"Research technogies to increase your technology points.\n\nUse the technology points to invent new items."
			  atPoint:NSMakePoint(xPos, yPos)
				width:300
			   height:400
			 fontSize:12
			   colour:[NSColor whiteColor]];
	
			
	yPos = 450 + 20;
	xPos = 400;
	
	[self displayText:@"Inventions"
			  atPoint:NSMakePoint(xPos, yPos)
			 fontSize:20
			   colour:[NSColor yellowColor]];
	
	yPos -= 2;
	yPos -= 20;
	
	if([thisInvention invented])
	{
		[self displayText:[thisInvention name]
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:20
				   colour:[NSColor whiteColor]];
		
		yPos -= 20;
		
		yPos -= 10;
		[self displayText:[thisInvention details]
				  atPoint:NSMakePoint(xPos, yPos)
					width:300
				   height:400
				 fontSize:12
				   colour:[NSColor whiteColor]];
	}
	else if([thisInvention invent])
	{
		[self displayText:[thisInvention name]
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:20
				   colour:[NSColor whiteColor]];
		
		yPos -= 20;
		
		[self displayText:[NSString stringWithFormat:@"%d turns until completion.", [thisInvention turnsToComplete]]
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		yPos -= 12;
		
		yPos -= 10;
		[self displayText:[thisInvention details]
				  atPoint:NSMakePoint(xPos, yPos)
					width:300
				   height:400
				 fontSize:12
				   colour:[NSColor whiteColor]];
	}
	else
	{
		
		
		
		[self displayText:[thisInvention name]
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:20
				   colour:[NSColor whiteColor]];
		
		yPos -= 20;
		
		[self displayText:@"Turns to invent:"
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		
		[self displayText:[NSString stringWithFormat:@"%d", [thisInvention turnsToComplete]]
				  atPoint:NSMakePoint(xPos + 100, yPos)
				 fontSize:12
				   colour:[NSColor whiteColor]];
		
		yPos -= 20;
		
		[self displayText:@"Requirements"
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		yPos -= 12;
		
		for(subLoop = 0; subLoop < TECH_COUNT; subLoop++)
		{
			if([thisInvention requiresTech:subLoop] > 0)
			{
				[self displayText:[game technologyName:subLoop]
						  atPoint:NSMakePoint(xPos, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				[self displayText:[NSString stringWithFormat:@"%d points", [thisInvention requiresTech:subLoop]]
						  atPoint:NSMakePoint(xPos + 80, yPos)
						 fontSize:12
						   colour:[NSColor whiteColor]];
				
				yPos -= 12;
			}
		}
		
		yPos -= 10;
		
		[self displayText:@"Existing Inventions Required"
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		yPos -= 12;
		
		if([thisInvention requirementCount] == 0)
		{
			[self displayText:@"None"
					  atPoint:NSMakePoint(xPos, yPos)
					 fontSize:12
					   colour:[NSColor whiteColor]];
			
			yPos -= 12;
		}
		else
		{
			for(loop = 0; loop < [thisInvention requirementCount]; loop++)
			{
				if([thisPlayer invented:[thisInvention getRequirementAtIndex:loop]])
				{
					[self displayText:[NSString stringWithFormat:@"%@ - invented", [thisInvention getRequirementAtIndex:loop]]
							  atPoint:NSMakePoint(xPos, yPos)
							 fontSize:12
							   colour:[NSColor whiteColor]];
				}
				else
				{
					[self displayText:[thisInvention getRequirementAtIndex:loop]
							  atPoint:NSMakePoint(xPos, yPos)
							 fontSize:12
							   colour:[NSColor whiteColor]];
				}
				yPos -= 12;
			}
		}
		
		yPos -= 10;
		
		if([game canPlayer:thisPlayer
					invent:thisInvention])
		{
			[self displayText:@"You can invent this."
					  atPoint:NSMakePoint(xPos, yPos)
					 fontSize:14
					   colour:[NSColor whiteColor]];
			yPos -= 16;
		}
		
		[self displayText:@"Details"
				  atPoint:NSMakePoint(xPos, yPos)
				 fontSize:12
				   colour:[NSColor yellowColor]];
		yPos -= 12;
		
		[self displayText:[thisInvention details]
				  atPoint:NSMakePoint(xPos, yPos)
					width:300
				   height:400
				 fontSize:12
				   colour:[NSColor whiteColor]];
	}
	
	// display the current task
	[self drawCurrentTask];
}

- (void)displayUnimplemented
{
	[[NSColor blackColor] set];
	
	NSRectFill(windowRect);
	
	[self displayText:@"Currently Unimplemented"
			  atPoint:NSMakePoint(0, 0)
			 fontSize:10
			   colour:[NSColor whiteColor]];
}

- (void)buttonChanged:(NSNotification*)notification
{	
	[self setNeedsDisplayInRect:NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, 100, 15)];
	[self setNeedsDisplayInRect:[buttonCurrent rect]];

	
	buttonCurrent = [notification object];

	[self setNeedsDisplayInRect:[buttonCurrent rect]];
	[self setNeedsDisplayInRect:NSMakeRect([buttonCurrent rect].origin.x + 5, [buttonCurrent rect].origin.y, 100, 15)];	
}

#pragma mark -
#pragma mark Icon Wheel Handling
- (void)addIconWheelItem:(id)newItem
{
	[iconWheelItemsArray addObject:newItem];
}

- (int)iconWheelCount;
{
	return [iconWheelItemsArray count];
}

- (id)getIconWheelItemAtIndex:(int)index
{
	return [iconWheelItemsArray objectAtIndex:index];
}

- (void)resetIconWheel
{
	[iconWheelItemsArray removeAllObjects];
}

#pragma mark -
#pragma mark Selection List Handling
- (void)addSelectionItem:(id)newItem
{
	[selectionItemsArray addObject:newItem];
}

- (int)selectionItemsCount
{
	return [selectionItemsArray count];
}

- (id)getSelectionItemAtIndex:(int)index
{
	return [selectionItemsArray objectAtIndex:index];
}

- (void)resetSelectionItems
{
	[selectionItemsArray removeAllObjects];
}

- (void)drawCurrentTask
{
	Task* task = [game currentTask];

	int yPos = 144;
	int xPos = 500; //502;
	int width = 300 - 2;
	int height = 100;
	
	if(task == nil)
	{
		return;
	}
	
	if([task completed])
	{
		return;
	}
	
	[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.9 alpha:1.0] colorWithAlphaComponent:0.8] set];
	
	NSRectFillUsingOperation(NSMakeRect(xPos, yPos - height, width, height), NSCompositeSourceOver);

	[[[NSColor colorWithDeviceRed:0.0 green:0.4 blue:0.4 alpha:1.0] colorWithAlphaComponent:0.8] set];
	
	NSRectFillUsingOperation(NSMakeRect(xPos + 2, yPos - height + 2, width - 4, height - 4), NSCompositeSourceOver);
	
	yPos -= 16;

	[self displayText:[NSString stringWithFormat:@"Training Task %d of %d", [game getCurrentTaskIndex], [game taskCount]]
			  atPoint:NSMakePoint(xPos + 4, yPos)
			 fontSize:10
			   colour:[NSColor whiteColor]];
	
	yPos -= 16;
	
	
	[self displayText:[task title]
			  atPoint:NSMakePoint(xPos + 4, yPos)
			 fontSize:14
			   colour:[NSColor whiteColor]];
	
	yPos -= 14;
	
	[self displayText:[task overview]
			  atPoint:NSMakePoint(xPos + 4, yPos)
				width:width - 6
			   height:50
			 fontSize:12
			   colour:[NSColor whiteColor]];
	

}

// load the graphics used by the game
- (void)loadGraphics
{
	imageWoodPanel = [NSImage imageNamed:@"wood_panel"];
	imageAboveWoodPanel = [NSImage imageNamed:@"above_wood_panel"];
	imageGrassTexture = [NSImage imageNamed:@"grass"];

}

- (void)setDragged:(id)newDrag
{
	dragged = newDrag;
}

- (id)dragged
{
	return dragged;
}

- (void)buildIconWheel:(id)item
{
	int loop;
	Player* thisPlayer = [game currentPlayer];
	Transport* thisTransport = nil;
	Factory* thisFactory = nil;
	Distance* thisDistance = nil;

	// if item of nil is passed then we use the existing parent item
	if(item == nil)
	{
		item = iconWheelParent;
	}
	else
	{
		iconWheelParent = item;
	}
	
	[self resetSelectionItems];
	
	[self addSelectionItem:item];
	
	if([item isKindOfClass:[City class]])
	{
		// add the transport at the current location
		for(loop = 0; loop < [thisPlayer transportCount]; loop++)
		{
			thisTransport = [thisPlayer getTransportAtIndex:loop];
			
			if([thisTransport location] == item)
			{
				[self addSelectionItem:thisTransport];
			}
		}
	}
	
	if([item isKindOfClass:[Factory class]])
	{
		thisFactory = item;
		
		// add the transport at the current location
		for(loop = 0; loop < [thisPlayer transportCount]; loop++)
		{
			thisTransport = [thisPlayer getTransportAtIndex:loop];
			
			if([thisTransport location] == item)
			{
				[self addSelectionItem:thisTransport];
			}
		}
	
		if([thisFactory turnsToBuild] == 0 && [thisFactory size] > 0)
		{
			// add factory upgrade options
			if([[thisPlayer getInventionCalled:@"Loom"] invented] && [thisFactory size] < [thisFactory sizeMax])
			{
				Action* newAction = [[[Action alloc] init] retain];
				[newAction setName:@"Upgrade Factory\nCost 100 per turn\nTakes 3 turns"];
				[newAction setProcess:@selector(upgradeFactory)];
				[newAction setParam1:nil];
				[newAction setDraggable:NO];
				[newAction setImage:[NSImage imageNamed:@"build_factory"]];				
				[self addSelectionItem:newAction];
			}
			
			// add buy cart option
			if([thisPlayer canAfford:100])
			{
				Action* newAction = [[[Action alloc] init] retain];
				[newAction setName:@"Buy Horse & Cart\nCost 100"];
				[newAction setProcess:@selector(buyTransport:)];
				[newAction setParam1:[NSNumber numberWithInt:TRANSPORT_CART]];
				[newAction setDraggable:NO];
				[newAction setImage:[NSImage imageNamed:@"buy_cart"]]; 
				[self addSelectionItem:newAction];
				
			}
			
			// add buy barge option
			if([[thisPlayer getInventionCalled:@"Canal"] invented] && [thisPlayer canAfford:500])
			{
				// check to see if there is a network leaving the city for the transport to run on
				BOOL built = NO;
				
				for(loop = 0; loop < [game distanceCount]; loop++)
				{
					thisDistance = [game getDistanceAtIndex:loop];
					
					if([thisDistance factory] == thisFactory && [thisDistance built:NETWORK_CANAL])
					{
						built = YES;
					}
				}
				
				if(built)
				{
					Action* newAction = [[[Action alloc] init] retain];
					[newAction setName:@"Buy Barge\nCost 500"];
					[newAction setProcess:@selector(buyTransport:)];
					[newAction setParam1:[NSNumber numberWithInt:TRANSPORT_BARGE]];
					[newAction setDraggable:NO];
					[newAction setImage:[NSImage imageNamed:@"buy_barge"]]; 
					[self addSelectionItem:newAction];
				}
			}
			
			// add buy train option
			if([[thisPlayer getInventionCalled:@"Rail"] invented] && [thisPlayer canAfford:1000])
			{
				// check to see if there is a network leaving the city for the transport to run on
				BOOL built = NO;
				
				for(loop = 0; loop < [game distanceCount]; loop++)
				{
					thisDistance = [game getDistanceAtIndex:loop];
					
					if([thisDistance factory] == thisFactory && [thisDistance built:NETWORK_RAIL])
					{
						built = YES;
					}
				}
				
				if(built)
				{
					Action* newAction = [[[Action alloc] init] retain];
					[newAction setName:@"Buy Train\nCost 1000"];
					[newAction setProcess:@selector(buyTransport:)];
					[newAction setParam1:[NSNumber numberWithInt:TRANSPORT_TRAIN]];
					[newAction setDraggable:NO];
					[newAction setImage:[NSImage imageNamed:@"buy_train"]]; 
					[self addSelectionItem:newAction];
				}
			}
		}
		
		if([thisFactory size] == 0)
		{
			Action* newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Build Tin Mine\nCost 100 per turn\nTakes 3 turns"];
			[newAction setProcess:@selector(buildFactory:)];
			[newAction setParam1:[NSNumber numberWithInt:FACTORY_TIN]];
			[newAction setDraggable:NO];
			[newAction setImage:[NSImage imageNamed:@"build_factory"]];

			[self addSelectionItem:newAction];
			
			newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Build Cotton Mill\nCost 100 per turn\nTakes 3 turns"];
			[newAction setProcess:@selector(buildFactory:)];
			[newAction setParam1:[NSNumber numberWithInt:FACTORY_COTTON]];
			[newAction setDraggable:NO];
			[newAction setImage:[NSImage imageNamed:@"build_factory"]];

			[self addSelectionItem:newAction];
			
			newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Build Pottery\nCost 100 per turn\nTakes 3 turns"];
			[newAction setProcess:@selector(buildFactory:)];
			[newAction setParam1:[NSNumber numberWithInt:FACTORY_POTTERY]];
			[newAction setDraggable:NO];
			[newAction setImage:[NSImage imageNamed:@"build_factory"]];

			[self addSelectionItem:newAction];
		}
		
		// add construction options
		if([[thisPlayer getInventionCalled:@"Rail"] invented])
		{
			Action* newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Build Railway\nCost 1000 per turn"];
			[newAction setProcess:@selector(commenceConstruction:)];
			[newAction setParam1:[NSNumber numberWithInt:NETWORK_RAIL]];
			[newAction setDraggable:YES];
			[newAction setImage:[NSImage imageNamed:@"build_railway"]];

			[self addSelectionItem:newAction];
		}
		
		if([[thisPlayer getInventionCalled:@"Canal"] invented])
		{
			Action* newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Build Canal\nCost 1000 per turn"];
			[newAction setProcess:@selector(commenceConstruction:)];
			[newAction setParam1:[NSNumber numberWithInt:NETWORK_CANAL]];
			[newAction setDraggable:YES];
			[newAction setImage:[NSImage imageNamed:@"build_canal"]];

			[self addSelectionItem:newAction];
		}
	}
	
	if([item isKindOfClass:[Transport class]])
	{
		// add load action
		if([game currentTransport] && [[game currentTransport] location] == [game currentFactory])
		{
			Action* newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Load Transport"];
			[newAction setProcess:@selector(loadTransport)];
			[newAction setParam1:nil];
			[newAction setDraggable:NO];
			[newAction setImage:[NSImage imageNamed:@"load"]]; 
			[self addSelectionItem:newAction];
		}
		
		// add unload action
		if([game currentTransport] && [[game currentTransport] location] == [game currentFactory])
		{
			Action* newAction = [[[Action alloc] init] retain];
			[newAction setName:@"Unload Transport"];
			[newAction setProcess:@selector(unloadTransport)];
			[newAction setParam1:nil];
			[newAction setDraggable:NO];
			[newAction setImage:[NSImage imageNamed:@"unload"]];
			[self addSelectionItem:newAction];
		}
		
		// add auto return action
		{
			Action* newAction = [[[Action alloc] init] retain];
			
			if([[game currentTransport] autoReturn])
			{
				[newAction setName:@"Turn Off Auto Return"];
				[newAction setProcess:@selector(autoReturnTransport:)];
				[newAction setParam1:[NSNumber numberWithInt:NO]];
				[newAction setDraggable:NO];
				[newAction setImage:[NSImage imageNamed:@"auto_return_off"]];
			}
			else
			{
				[newAction setName:@"Turn On Auto Return"];
				[newAction setProcess:@selector(autoReturnTransport:)];
				[newAction setParam1:[NSNumber numberWithInt:YES]];
				[newAction setDraggable:NO];
				[newAction setImage:[NSImage imageNamed:@"auto_return_on"]];
			}
			
			[self addSelectionItem:newAction];
		}
	}
}
@end
