//
//  Player.h
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import <Foundation/Foundation.h>

@class Factory;
@class Transport;
@class IRGame;
@class Invention;
@class Stats;

enum { TECH_CONSTRUCTION, TECH_STEAM, TECH_PUMP, TECH_ART, TECH_METAL, TECH_COUNT };

@interface Player : NSObject {
	NSString* name; // name of the player / character
	int ai;			// which AI to use, 0 - indicates human controlled
	int money;		// amount of money that the player has
	int researchBudget; // amount of research that can be carried out
	
	int profit;		// amount of money taken in the turn
	int loss;		// amount of moeny spent in the turn
	
	NSMutableArray* transportArray; // transport owned
	NSMutableArray* inventionArray; // inventions
	
	int tech[TECH_COUNT];  // array of technology levels for the player
	int research[TECH_COUNT];   // array of research turns remaining for each technology type
	
	NSMutableArray* statsArray; // array of stats information for previous turns
	
	IRGame* game;
}

- (void)setGame:(IRGame*)newGame;

-(void)setName:(NSString*)newName;
-(NSString*)name;

-(void)setAI:(int)newAI;
-(int)AI;

-(void)setMoney:(int)newMoney;
-(int)money;
-(void)changeMoney:(int)delta;
-(BOOL)canAfford:(int)amount;

-(void)addTransport:(Transport*)newTransport;
-(int)transportCount;
-(Transport*)getTransportAtIndex:(int)index;
-(int)transportCountOfType:(int)type;
-(void)removeTransport:(Transport*)thisTransport;

- (int)factoryCountOfType:(int)type;

-(void)setProfit:(int)newValue;
-(int)profit;
-(BOOL)changeProfit:(int)delta;

-(void)setLoss:(int)newValue;
-(int)loss;
-(BOOL)changeLoss:(int)delta;

-(void)setTech:(int)type level:(int)newLevel;
-(int)tech:(int)type;
-(void)changeTech:(int)type by:(int)delta;

-(void)setResearch:(int)type level:(int)newLevel;
-(int)research:(int)type;
-(BOOL)changeResearch:(int)type by:(int)delta;

-(void)addInvention:(Invention*)newInvention;
-(int)inventionCount;
-(Invention*)getInventionAtIndex:(int)index;
-(Invention*)getInventionCalled:(NSString*)searchName;
-(BOOL)invented:(NSString*)searchName;

-(void)setResearchBudget:(int)newValue;
-(int)researchBudget;
-(BOOL)changeResearchBudget:(int)delta;

-(void)addStats:(Stats*)newStats;
-(int)statsCount;
-(Stats*)getStatsAtIndex:(int)index;
-(Stats*)getLastStats;

@end
