//
//  Player.m
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import "Player.h"
#import "Transport.h"
#import "City.h"
#import "Distance.h"
#import "IRGame.h"
#import "Invention.h"


@implementation Player

- (id) init
{
	int loop;
	self = [super init];
	
	transportArray = [[NSMutableArray alloc] initWithCapacity:0];
	inventionArray = [[NSMutableArray alloc] initWithCapacity:0];
	statsArray = [[NSMutableArray alloc] initWithCapacity:0];
	
	// set technolgy levels to 0
	for(loop = 0; loop < TECH_COUNT; loop++)
	{
		[self setTech:loop level:0];
		[self setResearch:loop level:0];
	}
	
	[self setResearchBudget:1];
    return self;
}

- (void) dealloc
{	
		
	if(transportArray)
	{
		[transportArray release];
	}
	
	if(inventionArray)
	{
		[inventionArray release];
	}
	
	
	if(statsArray)
	{
		[statsArray release];
	}
	
    [super dealloc];
}

- (void)setGame:(IRGame*)newGame
{
	game = newGame;
}


- (void)setName:(NSString*)newName
{
	id old = name;
	name = [newName retain];
	[old release];	
}

- (NSString*)name
{
	return name;
}

-(void)setAI:(int)newAI
{
	ai = newAI;
}

-(int)AI
{
	return ai;
}

-(void)setMoney:(int)newMoney
{
	money = newMoney;
}

-(int)money
{
	return money;
}

-(void)changeMoney:(int)delta
{
	money += delta;
	
	if(delta > 0)
	{
		[self changeProfit:delta];
	}
	else
	{
		[self changeLoss:-delta];
	}
}

-(BOOL)canAfford:(int)amount
{
	if(money - amount >= 0)
	{
		return YES;
	}
	else
	{
		return NO;
	}
}




-(void)addTransport:(Transport*)newTransport
{
	[transportArray addObject:newTransport];
}

-(int)transportCount
{
	return [transportArray count];
}

-(Transport*)getTransportAtIndex:(int)index
{
	return [transportArray objectAtIndex:index];
}


-(int)transportCountOfType:(int)type
{
	int loop;
	int count = 0;
	Transport* thisTransport = nil;
	
	for(loop = 0; loop < [transportArray count]; loop++)
	{
		thisTransport = [transportArray objectAtIndex:loop];
		if([thisTransport type] == type)
		{
			if([thisTransport turnsToBuild] == 0)
			{
				count++;
			}
		}
	}
	
	return count;
}

-(void)removeTransport:(Transport*)thisTransport
{
	[transportArray removeObject:thisTransport];
}

- (int)factoryCountOfType:(int)type
{
	int loop;
	int count = 0;
	
	Factory* thisFactory = nil;
	
	for(loop = 0; loop < [game factoryCount]; loop++)
	{
		thisFactory = [game getFactoryAtIndex:loop];
		
		if([thisFactory type] == type && [thisFactory owner] == self && [thisFactory turnsToBuild] == 0)
		{
			count++;
		}
	}
	
	return count;
}

-(void)setProfit:(int)newValue
{
	profit = newValue;
}

-(int)profit
{
	return profit;
}

-(BOOL)changeProfit:(int)delta
{
	profit += delta;
	return YES;
}


-(void)setLoss:(int)newValue
{
	loss = newValue;
}

-(int)loss
{
	return loss;
}

-(BOOL)changeLoss:(int)delta
{
	loss += delta;
	return YES;
}


-(void)setTech:(int)type level:(int)newLevel
{
	tech[type] = newLevel;
}

-(int)tech:(int)type
{
	return tech[type];
}

-(void)changeTech:(int)type by:(int)delta
{
	tech[type] += delta;
}

-(void)setResearch:(int)type level:(int)newLevel
{
	research[type] = newLevel;
}

-(int)research:(int)type
{
	return research[type];
}

-(BOOL)changeResearch:(int)type by:(int)delta
{
	research[type] += delta;
	
	if(research[type] == 0)
	{
		return YES;
	}
	else
	{
		return NO;
	}
}


-(void)addInvention:(Invention*)newInvention
{
	[inventionArray addObject:newInvention];
}

-(int)inventionCount
{
	return [inventionArray count];
}

-(Invention*)getInventionAtIndex:(int)index
{
	return [inventionArray objectAtIndex:index];
}

-(Invention*)getInventionCalled:(NSString*)searchName
{
	int loop;
	Invention* thisInvention = nil;
	
	for(loop = 0; loop < [self inventionCount]; loop++)
	{
		thisInvention = [self getInventionAtIndex:loop];
		
		if([[thisInvention name] isEqualToString:searchName])
		{
			return thisInvention;
		}
	}
	
	return nil;
}

-(BOOL)invented:(NSString*)searchName
{
	Invention* thisInvention = nil;
	
	if([searchName isEqualToString:@""])
	{
		return YES;
	}
	
	thisInvention = [self getInventionCalled:searchName];
	
	if(thisInvention)
	{
		if([thisInvention turnsToComplete] == 0)
		{
			return [thisInvention invented];
		}
	}
	
	return NO;
}

-(void)setResearchBudget:(int)newValue
{
	researchBudget = newValue;
}

-(int)researchBudget
{
	return researchBudget;
}

-(BOOL)changeResearchBudget:(int)delta
{
	researchBudget += delta;
	return YES;
}

-(void)addStats:(Stats*)newStats
{
	[statsArray addObject:newStats];
}

-(int)statsCount
{
	return [statsArray count];
}

-(Stats*)getStatsAtIndex:(int)index
{
	return [statsArray objectAtIndex:index];
}

-(Stats*)getLastStats
{
	return [statsArray lastObject];
}


@end
