//
//  Transport.h
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import <Foundation/Foundation.h>

enum { TRANSPORT_CART, TRANSPORT_TRAIN, TRANSPORT_BARGE };

@class Distance;

@interface Transport : NSObject {
	int type;   // the type of transport
	int capacity; // max capicity of the transport
	int load;   // current load of the transport
	int cost;			// cost of the item

	
	id from;		// the location the transport departed from
	id destination; // the transport's destination - either a City or a Factory
	id location;	// present location of the transport
	int distanceRemaining;  // moves required to reach the destination
	int speed;		// movement rate for the transport
	
	int turnsToBuild;   // number of turns remaining until transport is built

	
	Distance* route; // the route the transport is travelling along
	BOOL autoReturn;	// does the transport automatically return and refill?
	
	NSImage* image;	// image to display the transport
	
	NSPoint positionMap;	// the on screen map location of the city
	NSPoint positionIconWheel;  // positon on the icon wheel
	
	NSString* invention;	// the invention required to buy the transport

}

- (void)setAutoReturn:(BOOL)flag;
- (BOOL)autoReturn;

-(void)setCost:(int)newCost;
-(int)cost;

-(void)setType:(int)newValue;
-(int)type;
- (NSString*)typeAsText;
- (NSString*)name;

-(void)setCapacity:(int)newValue;
-(int)capacity;
-(BOOL)changeCapacity:(int)delta;

-(void)setLoad:(int)newValue;
-(int)load;
-(BOOL)changeLoad:(int)delta;

- (void)setLocation:(id)newLocation;
- (id)location;
- (NSString*)locationName;

- (void)setDestination:(id)newDestination;
- (id)destination;
- (NSString*)destinationName;

- (void)setFrom:(id)newFrom;
- (id)from;
- (NSString*)fromName;

-(void)setDistanceRemaining:(int)newValue;
-(int)distanceRemaining;
-(BOOL)changeDistanceRemaining:(int)delta;

-(void)setSpeed:(int)newValue;
-(int)speed;
-(BOOL)changeSpeed:(int)delta;

- (void)setRoute:(Distance*)newRoute;
- (Distance*)route;

- (void)setTurnsToBuild:(int)newTurns;
- (int)turnsToBuild;
- (void)changeTurnsToBuild:(int)change;

- (void)setImage:(NSImage *)newImage;
- (NSImage*)image;

- (void)setPositionMap:(NSPoint)newPosition;
- (NSPoint)positionMap;

- (void)setPositionIconWheel:(NSPoint)newPosition;
- (NSPoint)positionIconWheel;

-(void)setInvention:(NSString*)newInvention;
-(NSString*)invention;


@end
