//
//  Transport.m
//  Industrial Revolution - a turn based business game.
//
//  Created by Andrew on 24/10/2004.
//  Copyright (C) 2004 Andrew Sage. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify  it under the terms of the GNU General Public
//  License as published by  the Free Software Foundation; either version 2 of the License.
//
//  This program is distributed in the hope that it will be useful,  but WITHOUT ANY WARRANTY; without even the implied
//  warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License  along with this program; if not, write to the
//  Free Software  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "Transport.h"


@implementation Transport

- (id) init
{
	self = [super init];
	
	[self setType:TRANSPORT_CART];
	[self setLoad:0];
	[self setCapacity:10];
	[self setDestination:nil];
	[self setLocation:nil];
	[self setFrom:nil];
	[self setRoute:nil];
	[self setAutoReturn:NO];
		
    return self;
}

-(void)setType:(int)newValue
{
	type = newValue;
	
	// set default speeds
	switch(type)
	{
		case TRANSPORT_CART:
			[self setImage:[NSImage imageNamed:@"cart"]];
			[self setSpeed:1];
			[self setCapacity:10];
			[self setInvention:@""];
			[self setCost:100];
			[self setTurnsToBuild:1];
			break;
			
		case TRANSPORT_TRAIN:
			[self setImage:[NSImage imageNamed:@"railway"]];
			[self setSpeed:4];
			[self setCapacity:60];
			[self setInvention:@"Rail"];
			[self setCost:1000];
			[self setTurnsToBuild:3];
			break;
			
		case TRANSPORT_BARGE:
			[self setSpeed:3];
			[self setImage:[NSImage imageNamed:@"barge"]];
			[self setCapacity:30];
			[self setInvention:@"Canal"];
			[self setCost:500];
			[self setTurnsToBuild:2];
			break;
	}
}

-(int)type
{
	return type;
}

- (NSString*)typeAsText
{
	switch(type)
	{
		case TRANSPORT_CART:
			return @"Horse and Cart";
			break;
			
		case TRANSPORT_TRAIN:
			return @"Train";
			break;
			
		case TRANSPORT_BARGE:
			return @"Barge";
			break;
			
		default:
			return @"Unknown";
			break;
	}
}

- (NSString*)name
{
	return [self typeAsText];
}

-(void)setCapacity:(int)newValue
{
	capacity = newValue;
}

-(int)capacity
{
	return capacity;
}

-(BOOL)changeCapacity:(int)delta
{
	capacity += delta;
	return YES;
}


-(void)setLoad:(int)newValue
{
	load = newValue;
}

-(int)load
{
	return load;
}

-(BOOL)changeLoad:(int)delta
{
	if(load + delta >= 0 && load + delta <= capacity)
	{
		load += delta;
		return YES;
	}
	
	return NO;
}

-(void)setDistanceRemaining:(int)newValue
{
	distanceRemaining = newValue;
}

-(int)distanceRemaining
{
	return distanceRemaining;
}

// returns YES if destination reached, NO if distance still to be travelled
-(BOOL)changeDistanceRemaining:(int)delta
{
	distanceRemaining += delta;
	
	if(distanceRemaining <= 0)
	{
		distanceRemaining = 0;
		return YES;
	}
	
	return NO;
}

- (void)setDestination:(id)newDestination
{
	destination = newDestination;
	
	// remember where we came from
	[self setFrom:[self location]];
}

- (id)destination
{
	return destination;
}

- (NSString*)destinationName
{
	if(destination != nil)
	{
		return [destination name];
	}
	
	return @"-";
}


- (void)setFrom:(id)newFrom
{
	from = newFrom;
}

- (id)from
{
	return from;
}

- (NSString*)fromName
{
	if(from != nil)
	{
		return [from name];
	}
	
	return @"-";
}

- (void)setLocation:(id)newLocation
{
	location = newLocation;
}

- (id)location
{
	return location;
}

- (NSString*)locationName
{
	if(location != nil)
	{
		return [location name];
	}
	
	return @"-";
}

-(void)setSpeed:(int)newValue
{
	speed = newValue;
}

-(int)speed
{
	return speed;
}

-(BOOL)changeSpeed:(int)delta
{
	speed += delta;
	return YES;
}

- (void)setRoute:(Distance*)newRoute
{
	route = newRoute;
}

- (Distance*)route
{
	return route;
}


- (void)setTurnsToBuild:(int)newTurns
{
	turnsToBuild = newTurns;
}

- (int)turnsToBuild
{
	return turnsToBuild;
}

- (void)changeTurnsToBuild:(int)change
{
	turnsToBuild += change;
}


- (void)setImage:(NSImage *)newImage
{
	image = newImage;
}

- (NSImage*)image
{
	return image;
}


- (void)setPositionMap:(NSPoint)newPosition
{
	positionMap = newPosition;
}

- (NSPoint)positionMap
{
	return positionMap;
}

- (void)setPositionIconWheel:(NSPoint)newPosition
{
	positionIconWheel = newPosition;
}

- (NSPoint)positionIconWheel
{
	return positionIconWheel;
}

- (void)setAutoReturn:(BOOL)flag
{
	autoReturn = flag;
}

- (BOOL)autoReturn
{
	return autoReturn;
}

- (void)setInvention:(NSString*)newInvention
{
	id old = invention;
	invention = [newInvention retain];
	[old release];	
}

- (NSString*)invention
{
	return invention;
}

-(void)setCost:(int)newCost
{
	cost = newCost;
}

-(int)cost
{
	return cost;
}
@end
